""" read data from pqsql;
    code --> name from table stock_list;
    swl_index --> name_level  from table swl_list;
    stock code belong to swl_level, L1,L2,L3 from table stock_belong_swl:
"""

import pandas as pd
from rrshare.rqUtil import read_data_from_pg, client_pgsql

def stock_code_to_name(code='',df=True):
    """ data get from local sql
        code --> 600519
        df=ture --> odataframe
        df=fasle --> dict 
    """
    stocks = read_data_from_pg(table_name='stock_list')[['code', 'name']]
    if code:
        if isinstance(code,str):
            code = [code]
        stock = stocks[stocks.code.isin(code)]
        if df:
            return stock
        return dict(zip(stock.code,stock.name))
    else:
        if df:
            return stocks
        return dict(zip(stocks.code,stocks.name))
        

def swl_index_to_name(index="", swl_level="", out_type=""):
    """index like 801020 not index_code 801020.SI
    swl_level= L1, L2, L3
        out_type =  dataframe, dict, list
    """
    swls = read_data_from_pg('swl_list')
    if swl_level:
        swls = swls[swls['level'] == swl_level][['index','name_level']]
    else:
        swls =swls[['index','name_level']]
    #print(swls)
    if index:
        if isinstance(index,list):
            swl = swls[swls['index'].isin(index)]
            swl_d = dict(zip(swl.index,swl.name_level))
            if out_type==dict:
                return swl_d
            if out_type==list:
                return list(swl_d.values())
            return swl
        else:
            return swls[swls['index'] == index]['name_level'].values[0]
    else:
        if out_type == dict:
            return dict(zip(swls.index,swls.name_level))
        return swls


def stock_code_belong_swl_name(code=None, swl_level=None):
    """ data get from local psql can select database--"rrshare"
        da change by client_pgsql("ttfactor") 
        swl_level --> L1 , L2, L3
        code --> 600519
        TODO
        #df=ture --> pd.dataframe
        #df=fasle --> dict 
    """
    stock_swls_all = read_data_from_pg(table_name='stock_belong_swl', client=client_pgsql("rrshare"))
    #print(stock_swls_all.head())
    #print(stock_swls_all.columns)
    if swl_level:
        swl_level_name = f"swl_{swl_level}"
        stock_swls = stock_swls_all[['code', 'name',swl_level_name]]
        #print(stock_swls)
    else:
        swl_level_name = ['swl_L1', 'swl_L2', 'swl_L3']
        cols =  ['code','name'] + swl_level_name
        #print(cols)
        stock_swls = stock_swls_all[cols]
        print(stock_swls.head())
    if code:
        if isinstance(code,str):
            code = [code]
        stock_swl = stock_swls[stock_swls.code.isin(code)]
        return stock_swl
    else:
        return stock_swls
       

if __name__ == '__main__':
    
    #print(stock_code_to_name('000831'))
    #print(stock_code_to_name(['000001','300674']))
    print(stock_code_to_name())
   
    #print(swl_index_to_name('850541'))
    print(swl_index_to_name(['850543', '850542']))
    print(swl_index_to_name(swl_level='L2'))
    
    print(stock_code_belong_swl_name(swl_level='L1'))
    #print(stock_code_belong_swl_name(code=["600519", "300146"]))
    #print(stock_code_belong_swl_name(code=['000001','600519'],swl_level='L3'))
    pass





import requests
from bs4 import BeautifulSoup
import pandas as pd

EastmoneyHeaders = {
        'Host': 'data.eastmoney.com',
        'User-Agent': 'Mozilla/5.0 (Windows NT 6.3; WOW64; Trident/7.0; Touch; rv:11.0) like Gecko',
        'Accept': '*/*',
        'Accept-Language': 'zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2',
        'Referer': 'http://data.eastmoney.com/bbsj/202112.html',
    }

def gen_secid(rawcode: str) -> str:
    '''
    生成东方财富专用的secid

    Parameters
    ----------
    rawcode : 6 位股票代码

    Return
    ------
    str: 指定格式的字符串

    '''
    # 沪市指数
    if rawcode[:3] == '000':
        return f'1.{rawcode}'
    # 深证指数
    if rawcode[:3] == '399':
        return f'0.{rawcode}'
    # 沪市股票
    if rawcode[0] != '6':
        return f'0.{rawcode}'
    # 深市股票
    return f'1.{rawcode}'

def fetch_stock_forecast_em(report_date="2021-06-30", **kwage):
    ''' report date
        df  
    '''
    day = '27'
    for p in range(1, 2):
        url = f"http://datacenter-web.eastmoney.com/securities/api/data/v1/get?\
            callback=jQuery112308107781430773868_1622071079060&\
                sortColumns=NOTICE_DATE%2CSECURITY_CODE&sortTypes=-1%2C-1&pageSize=5&\
                    pageNumber={p}&reportName=RPT_PUBLIC_OP_NEWPREDICT&columns=ALL&\
                        token=894050c76af8597a853f5b408b759f5d&filter=(REPORT_DATE%3D%{day}{report_date}%{day})"

        resp = requests.get(url)#, headers=EastmoneyHeaders)
        #print(resp)
        #print(resp.text)
        df = pd.DataFrame()
        #for i in range(1,10):
        resp_json = resp.json()['result']['data']
        print(resp_json)
        #df = df.append(pd.DataFrame.from_dict(resp_json, orient='index'))
        #print(df.T)
        #return(df.T)

if __name__ == "__main__":
    fetch_stock_forecast_em()


# coding=utf-8
import time
import pandas as pd
import easyquotation as eq 
from rrshare.rqUtil import rq_util_log_info
#from apscheduler.schedulers.blocking import BlockinfScheduler
#sched = BlockinfScheduler()

#quotation = eq.use('sina')  # 新浪 ['sina'] 腾讯 ['tencent', 'qq'] 
#获取所有股票行情
#snapshot_all = quotation.market_snapshot(prefix=True) 
# # prefix 参数指定返回的行情字典中的股票代码 key 是否带 sz/sh 前缀
#单只股票
#one_dict  = quotation.real('300146')
# 支持直接指定前缀，如 'sh000001'

def task_time(func_args): # TODO
    def in_func(*args, **kwargs):
        while True:
            start_time=time.time()
            data = func_args(*args, **kwargs)
            rq_util_log_info(time.time() - start_time)
            time.sleep(5)
        return data
    return in_func
    
    
def fetch_realtime_price_all(df=True,src='sina'): # or ['tecent','qq']
    quotation = eq.use(src)
    price_all = quotation.market_snapshot(prefix=True)
    if not df:
        return price_all
    price_df = pd.DataFrame(price_all).T.reset_index()
    return price_df

def fetch_etf_easyq(index_id="", min_volume=0, max_discount=None, min_discount=None):
    # TODO
    quotation = eq.use(['tecent', 'qq'])
    return quotation.etfindex()


def save_to_hdf(df,file_name='/tmp/snapshot_t1.h5'):
    while True:
        quotation = eq.use(src)
        price_all = quotation.market_snapshot(prefix=True)
        df = pd.DataFrame(price_all).T.reset_index()
        print(df[df['index']=='sz000001'][['index','name','now','time']])
        df.to_hdf(file_name, key='df', mode='w')
        rq_util_log_info(f'save df to hdf at {file_name}, ok')
        time.sleep(5)


def read_from_hdf(file_name='/tmp/snapshot_t1.h5'):
    df = pd.read_hdf(file_name)
    rq_util_log_info(df)
    return df


if __name__ =='__main__':
    
    df = fetch_realtime_price_all()
    print(df)
    #save_to_hdf(df)
    #read_from_hdf()
    pass




from rrshare.rqUpdate.update_stock_day import record_stock_day
from rrshare.rqUpdate.update_swl_day import record_swl_day
from rrshare.rqUpdate.update_swl_rs_valuation import record_swl_rs_valuation

from rrshare.rqUpdate.update_stock_RS_OH_MA import record_stock_PRS, record_stock_PRS_new







"""
输入参数

名称	类型	必选	描述
ts_code	str	Y	股票代码
ann_date	str	N	公告日期
start_date	str	N	公告开始日期
end_date	str	N	公告结束日期
period	str	N	报告期(每个季度最后一天的日期，比如20171231表示年报)
report_type	str	N	报告类型：见下方详细说明
comp_type	str	N	公司类型：1一般工商业 2银行 3保险 4证券
输出参数

名称	类型	默认显示	描述
ts_code	str	Y	TS股票代码
ann_date	str	Y	公告日期
f_ann_date	str	Y	实际公告日期
end_date	str	Y	报告期
report_type	str	Y	报表类型
comp_type	str	Y	公司类型
total_share	float	Y	期末总股本
cap_rese	float	Y	资本公积金
undistr_porfit	float	Y	未分配利润
surplus_rese	float	Y	盈余公积金
special_rese	float	Y	专项储备
money_cap	float	Y	货币资金
trad_asset	float	Y	交易性金融资产
notes_receiv	float	Y	应收票据
accounts_receiv	float	Y	应收账款
oth_receiv	float	Y	其他应收款
prepayment	float	Y	预付款项
div_receiv	float	Y	应收股利
int_receiv	float	Y	应收利息
inventories	float	Y	存货
amor_exp	float	Y	待摊费用
nca_within_1y	float	Y	一年内到期的非流动资产
sett_rsrv	float	Y	结算备付金
loanto_oth_bank_fi	float	Y	拆出资金
premium_receiv	float	Y	应收保费
reinsur_receiv	float	Y	应收分保账款
reinsur_res_receiv	float	Y	应收分保合同准备金
pur_resale_fa	float	Y	买入返售金融资产
oth_cur_assets	float	Y	其他流动资产
total_cur_assets	float	Y	流动资产合计
fa_avail_for_sale	float	Y	可供出售金融资产
htm_invest	float	Y	持有至到期投资
lt_eqt_invest	float	Y	长期股权投资
invest_real_estate	float	Y	投资性房地产
time_deposits	float	Y	定期存款
oth_assets	float	Y	其他资产
lt_rec	float	Y	长期应收款
fix_assets	float	Y	固定资产
cip	float	Y	在建工程
const_materials	float	Y	工程物资
fixed_assets_disp	float	Y	固定资产清理
produc_bio_assets	float	Y	生产性生物资产
oil_and_gas_assets	float	Y	油气资产
intan_assets	float	Y	无形资产
r_and_d	float	Y	研发支出
goodwill	float	Y	商誉
lt_amor_exp	float	Y	长期待摊费用
defer_tax_assets	float	Y	递延所得税资产
decr_in_disbur	float	Y	发放贷款及垫款
oth_nca	float	Y	其他非流动资产
total_nca	float	Y	非流动资产合计
cash_reser_cb	float	Y	现金及存放中央银行款项
depos_in_oth_bfi	float	Y	存放同业和其它金融机构款项
prec_metals	float	Y	贵金属
deriv_assets	float	Y	衍生金融资产
rr_reins_une_prem	float	Y	应收分保未到期责任准备金
rr_reins_outstd_cla	float	Y	应收分保未决赔款准备金
rr_reins_lins_liab	float	Y	应收分保寿险责任准备金
rr_reins_lthins_liab	float	Y	应收分保长期健康险责任准备金
refund_depos	float	Y	存出保证金
ph_pledge_loans	float	Y	保户质押贷款
refund_cap_depos	float	Y	存出资本保证金
indep_acct_assets	float	Y	独立账户资产
client_depos	float	Y	其中：客户资金存款
client_prov	float	Y	其中：客户备付金
transac_seat_fee	float	Y	其中:交易席位费
invest_as_receiv	float	Y	应收款项类投资
total_assets	float	Y	资产总计
lt_borr	float	Y	长期借款
st_borr	float	Y	短期借款
cb_borr	float	Y	向中央银行借款
depos_ib_deposits	float	Y	吸收存款及同业存放
loan_oth_bank	float	Y	拆入资金
trading_fl	float	Y	交易性金融负债
notes_payable	float	Y	应付票据
acct_payable	float	Y	应付账款
adv_receipts	float	Y	预收款项
sold_for_repur_fa	float	Y	卖出回购金融资产款
comm_payable	float	Y	应付手续费及佣金
payroll_payable	float	Y	应付职工薪酬
taxes_payable	float	Y	应交税费
int_payable	float	Y	应付利息
div_payable	float	Y	应付股利
oth_payable	float	Y	其他应付款
acc_exp	float	Y	预提费用
deferred_inc	float	Y	递延收益
st_bonds_payable	float	Y	应付短期债券
payable_to_reinsurer	float	Y	应付分保账款
rsrv_insur_cont	float	Y	保险合同准备金
acting_trading_sec	float	Y	代理买卖证券款
acting_uw_sec	float	Y	代理承销证券款
non_cur_liab_due_1y	float	Y	一年内到期的非流动负债
oth_cur_liab	float	Y	其他流动负债
total_cur_liab	float	Y	流动负债合计
bond_payable	float	Y	应付债券
lt_payable	float	Y	长期应付款
specific_payables	float	Y	专项应付款
estimated_liab	float	Y	预计负债
defer_tax_liab	float	Y	递延所得税负债
defer_inc_non_cur_liab	float	Y	递延收益-非流动负债
oth_ncl	float	Y	其他非流动负债
total_ncl	float	Y	非流动负债合计
depos_oth_bfi	float	Y	同业和其它金融机构存放款项
deriv_liab	float	Y	衍生金融负债
depos	float	Y	吸收存款
agency_bus_liab	float	Y	代理业务负债
oth_liab	float	Y	其他负债
prem_receiv_adva	float	Y	预收保费
depos_received	float	Y	存入保证金
ph_invest	float	Y	保户储金及投资款
reser_une_prem	float	Y	未到期责任准备金
reser_outstd_claims	float	Y	未决赔款准备金
reser_lins_liab	float	Y	寿险责任准备金
reser_lthins_liab	float	Y	长期健康险责任准备金
indept_acc_liab	float	Y	独立账户负债
pledge_borr	float	Y	其中:质押借款
indem_payable	float	Y	应付赔付款
policy_div_payable	float	Y	应付保单红利
total_liab	float	Y	负债合计
treasury_share	float	Y	减:库存股
ordin_risk_reser	float	Y	一般风险准备
forex_differ	float	Y	外币报表折算差额
invest_loss_unconf	float	Y	未确认的投资损失
minority_int	float	Y	少数股东权益
total_hldr_eqy_exc_min_int	float	Y	股东权益合计(不含少数股东权益)
total_hldr_eqy_inc_min_int	float	Y	股东权益合计(含少数股东权益)
total_liab_hldr_eqy	float	Y	负债及股东权益总计
lt_payroll_payable	float	Y	长期应付职工薪酬
oth_comp_income	float	Y	其他综合收益
oth_eqt_tools	float	Y	其他权益工具
oth_eqt_tools_p_shr	float	Y	其他权益工具(优先股)
lending_funds	float	Y	融出资金
acc_receivable	float	Y	应收款项
st_fin_payable	float	Y	应付短期融资款
payables	float	Y	应付款项
hfs_assets	float	Y	持有待售的资产
hfs_sales	float	Y	持有待售的负债
update_flag	str	N	更新标识
主要报表类型说明

代码	类型	说明
1	合并报表	上市公司最新报表（默认）
2	单季合并	单一季度的合并报表
3	调整单季合并表	调整后的单季合并报表（如果有）
4	调整合并报表	本年度公布上年同期的财务报表数据，报告期为上年度
5	调整前合并报表	数据发生变更，将原数据进行保留，即调整前的原数据
6	母公司报表	该公司母公司的财务报表数据
7	母公司单季表	母公司的单季度表
8	母公司调整单季表	母公司调整后的单季表
9	母公司调整表	该公司母公司的本年度公布上年同期的财务报表数据
10	母公司调整前报表	母公司调整之前的原始财务报表数据
11	调整前合并报表	调整之前合并报表原数据
12	母公司调整前报表	母公司报表发生变更前保留的原数据

"""

from rrshare.rqFetch import pro

df = pro.balancesheet(ts_code='600000.SH', start_date='20180101', end_date='20180730', \
    fields='ts_code,ann_date,f_ann_date,end_date,report_type,comp_type,cap_rese')

df2 = pro.balancesheet_vip(period='20181231',fields='ts_code,ann_date,f_ann_date,end_date,report_type,comp_type,cap_rese')

print(df2)


# coding: utf-8
from rrshare.rqUtil import rq_util_get_pre_trade_date
from rrshare.rqSU import (  #rq_save_stock_day_pg,
                         rq_save_stock_list_pg,
                         rq_save_stock_day_adj_fillna_pg)

def record_stock_day(N=255):
    #print(rq_util_get_pre_trade_date(n=255))
    rq_save_stock_list_pg()
    #rq_save_stock_day_pg()
    rq_save_stock_day_adj_fillna_pg(rq_util_get_pre_trade_date(n=N))

if __name__ == '__main__':
    record_stock_day(255)




"""输入参数

名称	类型	必选	描述
ts_code	str	Y	股票代码
ann_date	str	N	公告日期
start_date	str	N	公告开始日期
end_date	str	N	公告结束日期
period	str	N	报告期(每个季度最后一天的日期，比如20171231表示年报)
report_type	str	N	报告类型：见下方详细说明
comp_type	str	N	公司类型：1一般工商业 2银行 3保险 4证券
输出参数

名称	类型	默认显示	描述
ts_code	str	Y	TS股票代码
ann_date	str	Y	公告日期
f_ann_date	str	Y	实际公告日期
end_date	str	Y	报告期
comp_type	str	Y	公司类型
report_type	str	Y	报表类型
net_profit	float	Y	净利润
finan_exp	float	Y	财务费用
c_fr_sale_sg	float	Y	销售商品、提供劳务收到的现金
recp_tax_rends	float	Y	收到的税费返还
n_depos_incr_fi	float	Y	客户存款和同业存放款项净增加额
n_incr_loans_cb	float	Y	向中央银行借款净增加额
n_inc_borr_oth_fi	float	Y	向其他金融机构拆入资金净增加额
prem_fr_orig_contr	float	Y	收到原保险合同保费取得的现金
n_incr_insured_dep	float	Y	保户储金净增加额
n_reinsur_prem	float	Y	收到再保业务现金净额
n_incr_disp_tfa	float	Y	处置交易性金融资产净增加额
ifc_cash_incr	float	Y	收取利息和手续费净增加额
n_incr_disp_faas	float	Y	处置可供出售金融资产净增加额
n_incr_loans_oth_bank	float	Y	拆入资金净增加额
n_cap_incr_repur	float	Y	回购业务资金净增加额
c_fr_oth_operate_a	float	Y	收到其他与经营活动有关的现金
c_inf_fr_operate_a	float	Y	经营活动现金流入小计
c_paid_goods_s	float	Y	购买商品、接受劳务支付的现金
c_paid_to_for_empl	float	Y	支付给职工以及为职工支付的现金
c_paid_for_taxes	float	Y	支付的各项税费
n_incr_clt_loan_adv	float	Y	客户贷款及垫款净增加额
n_incr_dep_cbob	float	Y	存放央行和同业款项净增加额
c_pay_claims_orig_inco	float	Y	支付原保险合同赔付款项的现金
pay_handling_chrg	float	Y	支付手续费的现金
pay_comm_insur_plcy	float	Y	支付保单红利的现金
oth_cash_pay_oper_act	float	Y	支付其他与经营活动有关的现金
st_cash_out_act	float	Y	经营活动现金流出小计
n_cashflow_act	float	Y	经营活动产生的现金流量净额
oth_recp_ral_inv_act	float	Y	收到其他与投资活动有关的现金
c_disp_withdrwl_invest	float	Y	收回投资收到的现金
c_recp_return_invest	float	Y	取得投资收益收到的现金
n_recp_disp_fiolta	float	Y	处置固定资产、无形资产和其他长期资产收回的现金净额
n_recp_disp_sobu	float	Y	处置子公司及其他营业单位收到的现金净额
stot_inflows_inv_act	float	Y	投资活动现金流入小计
c_pay_acq_const_fiolta	float	Y	购建固定资产、无形资产和其他长期资产支付的现金
c_paid_invest	float	Y	投资支付的现金
n_disp_subs_oth_biz	float	Y	取得子公司及其他营业单位支付的现金净额
oth_pay_ral_inv_act	float	Y	支付其他与投资活动有关的现金
n_incr_pledge_loan	float	Y	质押贷款净增加额
stot_out_inv_act	float	Y	投资活动现金流出小计
n_cashflow_inv_act	float	Y	投资活动产生的现金流量净额
c_recp_borrow	float	Y	取得借款收到的现金
proc_issue_bonds	float	Y	发行债券收到的现金
oth_cash_recp_ral_fnc_act	float	Y	收到其他与筹资活动有关的现金
stot_cash_in_fnc_act	float	Y	筹资活动现金流入小计
free_cashflow	float	Y	企业自由现金流量
c_prepay_amt_borr	float	Y	偿还债务支付的现金
c_pay_dist_dpcp_int_exp	float	Y	分配股利、利润或偿付利息支付的现金
incl_dvd_profit_paid_sc_ms	float	Y	其中:子公司支付给少数股东的股利、利润
oth_cashpay_ral_fnc_act	float	Y	支付其他与筹资活动有关的现金
stot_cashout_fnc_act	float	Y	筹资活动现金流出小计
n_cash_flows_fnc_act	float	Y	筹资活动产生的现金流量净额
eff_fx_flu_cash	float	Y	汇率变动对现金的影响
n_incr_cash_cash_equ	float	Y	现金及现金等价物净增加额
c_cash_equ_beg_period	float	Y	期初现金及现金等价物余额
c_cash_equ_end_period	float	Y	期末现金及现金等价物余额
c_recp_cap_contrib	float	Y	吸收投资收到的现金
incl_cash_rec_saims	float	Y	其中:子公司吸收少数股东投资收到的现金
uncon_invest_loss	float	Y	未确认投资损失
prov_depr_assets	float	Y	加:资产减值准备
depr_fa_coga_dpba	float	Y	固定资产折旧、油气资产折耗、生产性生物资产折旧
amort_intang_assets	float	Y	无形资产摊销
lt_amort_deferred_exp	float	Y	长期待摊费用摊销
decr_deferred_exp	float	Y	待摊费用减少
incr_acc_exp	float	Y	预提费用增加
loss_disp_fiolta	float	Y	处置固定、无形资产和其他长期资产的损失
loss_scr_fa	float	Y	固定资产报废损失
loss_fv_chg	float	Y	公允价值变动损失
invest_loss	float	Y	投资损失
decr_def_inc_tax_assets	float	Y	递延所得税资产减少
incr_def_inc_tax_liab	float	Y	递延所得税负债增加
decr_inventories	float	Y	存货的减少
decr_oper_payable	float	Y	经营性应收项目的减少
incr_oper_payable	float	Y	经营性应付项目的增加
others	float	Y	其他
im_net_cashflow_oper_act	float	Y	经营活动产生的现金流量净额(间接法)
conv_debt_into_cap	float	Y	债务转为资本
conv_copbonds_due_within_1y	float	Y	一年内到期的可转换公司债券
fa_fnc_leases	float	Y	融资租入固定资产
end_bal_cash	float	Y	现金的期末余额
beg_bal_cash	float	Y	减:现金的期初余额
end_bal_cash_equ	float	Y	加:现金等价物的期末余额
beg_bal_cash_equ	float	Y	减:现金等价物的期初余额
im_n_incr_cash_equ	float	Y	现金及现金等价物净增加额(间接法)
update_flag	str	N	更新标识
主要报表类型说明

代码	类型	说明
1	合并报表	上市公司最新报表（默认）
2	单季合并	单一季度的合并报表
3	调整单季合并表	调整后的单季合并报表（如果有）
4	调整合并报表	本年度公布上年同期的财务报表数据，报告期为上年度
5	调整前合并报表	数据发生变更，将原数据进行保留，即调整前的原数据
6	母公司报表	该公司母公司的财务报表数据
7	母公司单季表	母公司的单季度表
8	母公司调整单季表	母公司调整后的单季表
9	母公司调整表	该公司母公司的本年度公布上年同期的财务报表数据
10	母公司调整前报表	母公司调整之前的原始财务报表数据
11	调整前合并报表	调整之前合并报表原数据
12	母公司调整前报表	母公司报表发生变更前保留的原数据

"""

from rrshare.rqFetch import pro

df = pro.cashflow(ts_code='600000.SH', start_date='20180101', end_date='20180730')

df2 = pro.cashflow_vip(period='20201231',fields='')

print(df2)


import time
import pandas as pd

from rrshare.rqUtil import rq_util_get_last_tradedate

from rrshare.rqFetch import  pro
#from rrshare.rqUtil import setting
from rrshare.rqUtil import (rq_util_code_tosrccode,rq_util_code_tostr)
#from rrshare.rqFetch.rqCodeName import (swl_index_to_name, stock_code_to_name)
from rrshare.rqUtil import PgsqlClass
from rrshare.rqFetch import Swsindex

conn = PgsqlClass().client_pg('rrshare')

class SwlIndexStock(object):
    """ first you need update -- stock_list
    then  save _swl_list;
    -->> save_swl_contains_stock
    -->> save_stock_belong_swl_L
    -->> save_stock_belong_swl_all

    """

    def __init__(self):
        #self.level = level
        self.LEVEL = ['L1','L2','L3']
        #self.symbol_to_tscode = lambda x: "".join([x,'.SH'] if str(x) > "333333" else "".join([x,'.SZ'])) 
        pass
    
    def all_stock_list_code(self,trade_date=rq_util_get_last_tradedate(), code_to_symbol=False):
        trade_date = str(trade_date).replace('-','')
        df = pro.adj_factor(trade_date=trade_date)
        if code_to_symbol:
            return list(df['ts_code'].apply(lambda x: x[0:6]).values)
        return list(df['ts_code'])

    
    def fetch_swl_index(self,level=""):
        df_tspro = pro.index_classify(level=level,src='SW2021')   #update
        df_tspro['index'] = df_tspro['index_code'].map(lambda x: x[:6])
        df_tspro['name_level'] = df_tspro['industry_name'] + "_" + df_tspro['level']
        print("get_swl_index")
        time.sleep(10)
        return df_tspro


    def save_swl_list(self):
        df_swl = self.fetch_swl_index()
        PgsqlClass().insert_to_psql(df_swl, 'rrshare','swl_list',if_exists='replace')


    def swl_index_to_name_dict(self,level=""):
        df = self.fetch_swl_index(level)
        return dict(zip(df['index_code'].values,df['name_level'].values))


    def stock_belong_swl_one(self,symbol):
        swl_index_code = self.fetch_swl_index()['index_code'].values
        fun = lambda x: "".join([x,'.SH'] if str(x) > "333333" else "".join([x,'.SZ'])) 
        ts_code = fun(symbol)
        df = pro.index_member(ts_code=ts_code)
        df = df[df.index_code.isin(swl_index_code)]
        df['name_level'] = df['index_code'].apply(lambda x : self.swl_index_to_name_dict()[x])

        return {ts_code : df['name_level'].values.tolist()}

    def swl_contains_stock(self):
        swl_all = pd.DataFrame()
        for level in self.LEVEL:
            #print(level)
            for l in self.fetch_swl_index(level)['index_code'].values:
                data = pro.index_member(index_code=l, fields='index_code, index_name,con_code, con_name, in_date')
                #print(data)
                swl_all = swl_all.append(data)
                swl_all['name_level'] = swl_all['index_name'] + "_" + str(level)
            #print(data)
        return swl_all

    def save_swl_contains_stock(self):
        swl_all = pd.DataFrame()
        for level in self.LEVEL:
            swl_l = pd.DataFrame()
            #print(level)
            for l in self.fetch_swl_index(level)['index_code'].values:
                data = pro.index_member(index_code=l, fields='index_code, index_name,con_code, con_name, in_date')
                #print(data)
                swl_l = swl_l.append(data)
                swl_l['name_level'] = swl_l['index_name'] + "_" + str(level)
            swl_l['level'] = level
            swl_all = swl_all.append(swl_l)
        PgsqlClass().insert_to_psql(swl_all,'rrshare','swl_contains_stock',if_exists='replace')
           

    def read_swl_contains_stock(self):
        from rrshare.rqUtil import PgsqlClass
        conn = PgsqlClass().client_pg('rrshare')
        df = PgsqlClass().read_psql('rrshare','swl_contains_stock')
        print(df)
        return df   


    def save_stock_belong_swl_L(self):
        df = self.read_swl_contains_stock()
        df = df.sort_values(by=['con_code'])
        df = df[['level', 'con_code','name_level']] #,'index_code']]
       
        for i,j in enumerate(["L1","L2","L3"]):
            #print(i, j)
            df_l = df[df.level == j]
            df_l.sort_values(by='con_code', inplace=True)
            df_l.rename(columns={'con_code':'ts_code'},inplace=True)
            df_l.set_index('ts_code', inplace=True)
            df_l.drop(columns=['level'], inplace=True)
            df_l.rename(columns={'name_level':f'swl_{j}'},inplace=True)
            df_l.reset_index(inplace=True)
            PgsqlClass().insert_to_psql(df_l, 'rrshare',f'stock_belong_swl_{j}',if_exists='replace')
    
        
    def save_stock_belong_swl_all(self):
        df0 = PgsqlClass().read_psql('rrshare','stock_list')[['ts_code','name','code']]
        df1 = PgsqlClass().read_psql('rrshare','stock_belong_swl_L1')
        df2 = PgsqlClass().read_psql('rrshare','stock_belong_swl_L2')
        df3= PgsqlClass().read_psql('rrshare','stock_belong_swl_L3')
        df_swl = pd.merge(df0,df1, how='left', on='ts_code')  
        df_swl = df_swl.merge(df2, how='left', on='ts_code')
        df_swl = df_swl.merge(df3, how='left', on='ts_code')
        #print(df_swl)
        PgsqlClass().insert_to_psql(df_swl, 'rrshare','stock_belong_swl',if_exists='replace')


def save_stock_belong_swsindex():
    swi = Swsindex()
    df_sws = swi.get_swl1_class()
    PgsqlClass().insert_to_psql(df_sws, 'rrshare','stock_belong_swl1',if_exists='replace')


def rq_save_swl_industry_list_stock_belong():
    swl =  SwlIndexStock()
    swl.save_swl_list()
    time.sleep(20)
    swl.save_swl_contains_stock()
    time.sleep(20)
    swl.save_stock_belong_swl_L()
    time.sleep(20)
    swl.save_stock_belong_swl_all()
    time.sleep(20)
    save_stock_belong_swsindex()
    

if __name__ == '__main__':
    #li = SwlIndexStock()
    #print(li.fetch_swl_index())
    #li.save_swl_list()
    #print(li.swl_index_to_name_dict())
    #print(li.stock_belong_swl('600196'))
    #df = li.swl_contains_stock()
    #print(df)
    #li.save_swl_contains_stock()
    #li.read_swl_contains_stock()
    #li.stock_belong_swl_pro()
    #li.save_stock_belong_swl_all()

    rq_save_swl_industry_list_stock_belong()
    pass




"""输入参数
名称	类型	必选	描述
ts_code	str	Y	股票代码
period	str	N	报告期(每个季度最后一天的日期,比如20171231表示年报)
type	str	N	类型：P按产品 D按地区（请输入大写字母P或者D）
start_date	str	N	报告期开始日期
end_date	str	N	报告期结束日期

输出参数
名称	类型	描述
ts_code	str	TS代码
end_date	str	报告期
bz_item	str	主营业务来源
bz_sales	float	主营业务收入(元)
bz_profit	float	主营业务利润(元)
bz_cost	float	主营业务成本(元)
curr_type	str	货币代码
update_flag	str	是否更新

"""
from rrshare.rqFetch import pro

df = pro.fina_mainbz_vip(ts_code='000627.SZ', type='P')
print(df)

all = pro.fina_mainbz_vip(ts_code='002236.SZ',period='20201231', type= 'D')
print(all)


    
from rrshare.rqFactor.swl_rs_valuation_pg import (update_swl_rs_valuation,update_swl_all_main, swl1_index_realtime_prs)

from rrshare.rqFactor.stock_RS_OH_MA import (stock_RT_HH_MA_pre, stock_RT_HH_MA_last, stock_RS_OH_MA_new, stock_select_PRS_new)




"""
这里定义的是一些常用常量
"""
from enum import Enum
from pytz import timezone

CHINA_TZ = timezone("Asia/Shanghai")

MAX_QUERY_SIZE: int = 5000
TS_DATE_FORMATE: str = '%Y%m%d'
MAX_QUERY_TIMES: int = 500

class TRADE_TIMES():
    TRADE_TIME_AM = ['09:30:00','11:30:00']
    TRADE_TIME_PM = ["13:00:00","15:00:00"]
    TRADE_TIME_AM_pre = ['09:25:00','11:31:00']
    TRADE_TIME_PM_pre = ["13:00:00","15:01:00"]

#print(TRADE_TIMES().TRADE_TIME_AM)

class REPORT_PERIODS():
    q1 = '0331'
    q2 = '0630'
    q3 = '0930'
    q4 = '1231'
    REPORT_PERIODS_DICT = { 'q1': '0331',
            'q2': '0630',
            'q3': '0930',
            'q4': '1231'}
    
class PERIODS():
    """stock trade period
    """
    PERIODS=[5,10,20,60,120,250]

class SWL_LEVEL():
    LEVEL =['L1','L2','L3']
    SWL_LEVEL = ['sw_l1','sw_l2','sw_l3']

class ORDER_DIRECTION():
    """订单的买卖方向
    BUY 股票 买入
    SELL 股票 卖出
    BUY_OPEN 期货 多开
    BUY_CLOSE 期货 空平(多头平旧仓)
    SELL_OPEN 期货 空开
    SELL_CLOSE 期货 多平(空头平旧仓)
    ASK  申购
    """

    BUY = 1
    SELL = -1
    BUY_OPEN = 2
    BUY_CLOSE = 3
    SELL_OPEN = -2
    SELL_CLOSE = -3
    SELL_CLOSETODAY = -4
    BUY_CLOSETODAY = 4
    ASK = 0
    XDXR = 5
    OTHER = 6


class TIME_CONDITION():
    IOC = 'IOC'  # 立即完成，否则撤销
    GFS = 'GFS'  # 本节有效
    GFD = 'GFD'  # 当日有效
    GTD = 'GTD'  # 指定日期前有效
    GTC = 'GTC'  # 撤销前有效
    GFA = 'GFA'  # 集合竞价有效


class VOLUME_CONDITION():
    ANY = 'ANY'  # 任意数量
    MIN = 'MIN'  # 最小数量
    ALL = 'ALL'  # 全部数量


class EXCHANGE_ID():
    SSE = 'sse'  # 上交所
    SZSE = 'szse'  # 深交所
    SHFE = 'SHFE'  # 上期所
    DCE = 'DCE'  # 大商所
    CZCE = 'CZCE'  # 郑商所
    CFFEX = 'CFFEX'  # 中金所
    INE = 'INE'  # 能源中心
    HUOBI = 'huobi' # 火币Pro
    BINANCE = 'binance' # 币安
    BITMEX = 'bitmex' # BITMEX
    BITFINEX = 'BITFINEX' # BITFINEX
    OKEX = 'OKEx' # OKEx


class OFFSET():
    """订单的开平仓属性
    OPEN 股票/期货 开仓
    CLOSE 股票 卖出
    CLOSE_HISTORY 期货 平昨
    CLOSE_TODAY 期货 平今
    REVERSE 期货 反手(默认先平昨再平今)
    """

    OPEN = 'OPEN'
    CLOSE = 'CLOSE'
    CLOSETODAY = 'CLOSETODAY'
    REVERSE = 'REVERSE'


class ORDER_MODEL():
    """订单的成交模式
    LIMIT 限价模式
    MARKET 市价单 # 目前市价单在回测中是bar的开盘价 /实盘里面是五档剩余最优成交价
    CLOSE 收盘单 # 及在bar的收盘时的价格
    NEXT_OPEN 下一个bar的开盘价成交
    STRICT 严格订单 不推荐/仅限回测/是在当前bar的最高价买入/当前bar的最低价卖出
    @yutiansut/2017-12-18
    """

    LIMIT = 'LIMIT'  # 限价
    ANY = 'ANY'  # 市价(otg兼容)
    MARKET = 'MARKET'  # 市价/在回测里是下个bar的开盘价买入/实盘就是五档剩余最优成交价
    CLOSE = 'CLOSE'  # 当前bar的收盘价买入
    NEXT_OPEN = 'NEXT_OPEN'  # 下个bar的开盘价买入
    STRICT = 'STRICT'  # 严格模式/不推荐(仅限回测测试用)
    BEST = 'BEST'  # 中金所  最优成交剩余转限
    FIVELEVEL = 'FIVELEVEL'


class ORDER_STATUS():
    """订单状态
    status1xx 订单待生成
    status3xx 初始化订单  临时扣除资产(可用现金/可卖股份)
    status3xx 订单存活(等待交易)
    status2xx 订单完全交易/未完全交易
    status4xx 主动撤单
    status500 订单死亡(每日结算) 恢复临时资产    
    订单生成(100) -- 进入待成交队列(300) -- 完全成交(200) -- 每日结算(500)-- 死亡
    订单生成(100) -- 进入待成交队列(300) -- 部分成交(203) -- 未成交(300) -- 每日结算(500) -- 死亡
    订单生成(100) -- 进入待成交队列(300) -- 主动撤单(400) -- 每日结算(500) -- 死亡
    """

    # NEW = 100
    # SUCCESS_ALL = 200
    # SUCCESS_PART = 203
    # QUEUED = 300  # queued 用于表示在order_queue中 实际表达的意思是订单存活 待成交
    # CANCEL_ALL = 400
    # CANCEL_PART = 402
    # SETTLED = 500
    # FAILED = 600

    NEW = 'new'
    SUCCESS_ALL = 'success_all'  # == FINISHED
    SUCCESS_PART = 'success_part'
    QUEUED = 'queued'  # queued 用于表示在order_queue中 实际表达的意思是订单存活 待成交 == ALIVED
    CANCEL_ALL = 'cancel_all'
    CANCEL_PART = 'cancel_part'
    SETTLED = 'settled'
    FAILED = 'failed'
    NEXT = 'next'  # 当前bar未成交,下一个bar继续等待


class AMOUNT_MODEL():
    """订单的成交量
    by_money是按固定成交总额下单,动态计算成交量
    by_amount 按固定数量下单
    """

    BY_MONEY = 'by_money'
    BY_AMOUNT = 'by_amount'


class RUNNING_ENVIRONMENT():
    """执行环境
    回测
    模拟
    t0
    实盘
    随机(按算法/分布随机生成行情)/仅用于训练测试
    """

    BACKETEST = 'backtest'
    SIMULATION = 'simulation'
    TZERO = 't0'
    REAL = 'real'
    RANDOM = 'random'
    TTS = 'tts'


class TRADE_STATUS():
    """交易状态返回值
    涨跌停限制: 202
    成功交易 : 200
    当天无交易数据: 500
    订单失败(比如买卖价格超过涨跌停价格范围,交易量过大等等):400
    """

    SUCCESS = 'trade_success'
    PRICE_LIMIT = 'trade_price_limit'  # 只是未成交
    NO_MARKET_DATA = 'trade_no_market_data'
    FAILED = 'trade_failed'


class MARKET_ERROR():
    """市场类的错误
    1. 账户以及存在(不能重复注册)
    2. 网络中断
    3. 数据库连接丢失
    4. 数值/索引不存在
    """

    ACCOUNT_EXIST = 'ACCOUNT EXIST {}'
    NETWORK_BROKERN = 'NETWORK BROKEN {}'
    DATABSECONNECT_LOST = 'DATABASECONNECTION LOST {}'
    VALUE_NOT_FOUND = 'VALUE_NOT_FOUND'


class MARKET_TYPE():
    """市场种类
    日线 尾数01
    分钟线 尾数02
    tick 尾数03
    市场:
    股票 0
    指数/基金 1
    期货 2
    港股 3
    美股 4
    比特币/加密货币市场 5
    """
    STOCK_CN = 'stock_cn'  # 中国A股
    STOCK_CN_B = 'stock_cn_b'  # 中国B股
    STOCK_CN_D = 'stock_cn_d'  # 中国D股 沪伦通
    STOCK_HK = 'stock_hk'  # 港股
    STOCK_US = 'stock_us'  # 美股
    FUTURE_CN = 'future_cn'  # 国内期货
    OPTION_CN = 'option_cn'  # 国内期权
    STOCKOPTION_CN = 'stockoption_cn'  # 个股期权
    # BITCOIN = 'bitcoin'  # 比特币
    CRYPTOCURRENCY = 'cryptocurrency'  # 加密货币(衍生货币)
    INDEX_CN = 'index_cn'  # 中国指数
    FUND_CN = 'fund_cn'   # 中国基金
    BOND_CN = 'bond_cn'  # 中国债券


class BROKER_TYPE():
    """执行环境
    回测
    模拟
    实盘
    随机(按算法/分布随机生成行情)/仅用于训练测试
    """

    BACKETEST = 'backtest'
    SIMULATION = 'simulation'
    REAL = 'real'
    RANDOM = 'random'
    SHIPANE = 'shipane'
    TTS = 'tts'


class EVENT_TYPE():
    """[summary]
    """

    BROKER_EVENT = 'broker_event'
    ACCOUNT_EVENT = 'account_event'
    MARKET_EVENT = 'market_event'
    TRADE_EVENT = 'trade_event'
    ENGINE_EVENT = 'engine_event'
    ORDER_EVENT = 'order_event'


class MARKET_EVENT():
    """交易前置事件
    query_order 查询订单
    query_assets 查询账户资产
    query_account 查询账户
    query_cash 查询账户现金
    query_data 请求数据
    query_deal 查询成交记录
    query_position 查询持仓
    """

    QUERY_ORDER = 'query_order'
    QUERY_ASSETS = 'query_assets'
    QUERY_ACCOUNT = 'query_account'
    QUERY_CASH = 'query_cash'
    QUERY_DATA = 'query_data'
    QUERY_DEAL = 'query_deal'
    QUERY_POSITION = 'query_position'


class ENGINE_EVENT():
    """引擎事件"""
    MARKET_INIT = 'market_init'
    UPCOMING_DATA = 'upcoming_data'
    UPCOMING_TICK = 'upcoming_tick'
    UPCOMING_BAR = 'upcoming_bar'
    BAR_SETTLE = 'bar_settle'
    DAILY_SETTLE = 'daily_settle'
    UPDATE = 'update'
    TRANSACTION = 'transaction'
    ORDER = 'order'


class ACCOUNT_EVENT():
    """账户事件"""
    UPDATE = 'account_update'
    SETTLE = 'account_settle'
    MAKE_ORDER = 'account_make_order'


class BROKER_EVENT():
    """BROKER事件
    BROKER 
    有加载数据的任务 load data
    撮合成交的任务 broker_trade
    轮询是否有成交记录 query_deal
    """
    LOAD_DATA = 'load_data'
    TRADE = 'broker_trade'
    SETTLE = 'broker_settle'
    DAILY_SETTLE = 'broker_dailysettle'
    RECEIVE_ORDER = 'receive_order'
    QUERY_DEAL = 'query_deal'
    NEXT_TRADEDAY = 'next_tradeday'


class ORDER_EVENT():
    """订单事件
    创建订单 create
    交易 trade
    撤单 cancel
    """
    CREATE = 'create'
    TRADE = 'trade'
    CANCEL = 'cancel'
    FAIL = 'fail'


class FREQUENCE():
    """查询的级别
    YEAR = 'year'  # 年bar
    QUARTER = 'quarter'  # 季度bar
    MONTH = 'month'  # 月bar
    WEEK = 'week'  # 周bar
    DAY = 'day'  # 日bar
    ONE_MIN = '1min'  # 1min bar
    FIVE_MIN = '5min'  # 5min bar
    FIFTEEN_MIN = '15min'  # 15min bar
    THIRTY_MIN = '30min'  # 30min bar
    HOUR = '60min'  # 60min bar
    SIXTY_MIN = '60min'  # 60min bar
    TICK = 'tick'  # transaction
    ASKBID = 'askbid'  # 上下五档/一档
    REALTIME_MIN = 'realtime_min' # 实时分钟线
    LATEST = 'latest'  # 当前bar/latest
    2019/08/06 @yutiansut
    """

    YEAR = 'year'  # 年bar
    QUARTER = 'quarter'  # 季度bar
    MONTH = 'month'  # 月bar
    WEEK = 'week'  # 周bar
    DAY = 'day'  # 日bar
    ONE_MIN = '1min'  # 1min bar
    FIVE_MIN = '5min'  # 5min bar
    FIFTEEN_MIN = '15min'  # 15min bar
    THIRTY_MIN = '30min'  # 30min bar
    HOUR = '60min'  # 60min bar
    SIXTY_MIN = '60min'  # 60min bar
    TICK = 'tick'  # transaction
    ASKBID = 'askbid'  # 上下五档/一档
    REALTIME_MIN = 'realtime_min'  # 实时分钟线
    LATEST = 'latest'  # 当前bar/latest


class CURRENCY_TYPE():
    """货币种类"""
    RMB = 'rmb'  # 人民币
    USD = 'usd'  # 美元
    EUR = 'eur'  # 欧元
    HKD = 'hkd'  # 港币
    GBP = 'GBP'  # 英镑
    BTC = 'btc'  # 比特币
    JPY = 'jpy'  # 日元
    AUD = 'aud'  # 澳元
    CAD = 'cad'  # 加拿大元


class DATASOURCE():
    """数据来源
    """

    WIND = 'wind'  # wind金融终端
    TDB = 'tdb'  # wind tdb
    THS = 'ths'  # 同花顺网页
    TUSHARE = 'tushare'  # tushare
    TDX = 'tdx'  # 通达信
    MONGO = 'mongo'  # 本地/远程Mongodb
    EASTMONEY = 'eastmoney'  # 东方财富网
    CHOICE = 'choice'  # choice金融终端
    CCXT = 'ccxt'  # github/ccxt 虚拟货币
    LOCALFILE = 'localfile'  # 本地文件
    AUTO = 'auto'  # 优先从Mongodb中读取数据，不足的数据从tdx下载


class OUTPUT_FORMAT():
    """输出格式
    """

    DATASTRUCT = 'datastruct'
    DATAFRAME = 'dataframe'
    SERIES = 'series'
    NDARRAY = 'ndarray'
    LIST = 'list'
    JSON = 'json'


class RUNNING_STATUS():
    """运行状态
    starting 是一个占用状态
    100 - 202 - 200 - 400 - 500
    """

    PENDING = 100
    SUCCESS = 200
    STARTING = 202
    RUNNING = 300
    WRONG = 400
    STOPED = 500
    DROPED = 600


DATABASE_TABLE = {
    (MARKET_TYPE.STOCK_CN, FREQUENCE.DAY): 'stock_day',
    (MARKET_TYPE.STOCK_CN, FREQUENCE.ONE_MIN): 'stock_min',
    (MARKET_TYPE.STOCK_CN, FREQUENCE.FIVE_MIN): 'stock_min',
    (MARKET_TYPE.STOCK_CN, FREQUENCE.FIFTEEN_MIN): 'stock_min',
    (MARKET_TYPE.STOCK_CN, FREQUENCE.THIRTY_MIN): 'stock_min',
    (MARKET_TYPE.STOCK_CN, FREQUENCE.SIXTY_MIN): 'stock_min',
    (MARKET_TYPE.STOCK_CN, FREQUENCE.HOUR): 'stock_min',
    (MARKET_TYPE.STOCK_CN, FREQUENCE.TICK): 'stock_transaction',
    (MARKET_TYPE.INDEX_CN, FREQUENCE.DAY): 'index_day',
    (MARKET_TYPE.INDEX_CN, FREQUENCE.ONE_MIN): 'index_min',
    (MARKET_TYPE.INDEX_CN, FREQUENCE.FIVE_MIN): 'index_min',
    (MARKET_TYPE.INDEX_CN, FREQUENCE.FIFTEEN_MIN): 'index_min',
    (MARKET_TYPE.INDEX_CN, FREQUENCE.THIRTY_MIN): 'index_min',
    (MARKET_TYPE.INDEX_CN, FREQUENCE.SIXTY_MIN): 'index_min',
    (MARKET_TYPE.INDEX_CN, FREQUENCE.HOUR): 'index_min',
    (MARKET_TYPE.INDEX_CN, FREQUENCE.TICK): 'index_transaction',
    (MARKET_TYPE.FUND_CN, FREQUENCE.DAY): 'index_day',
    (MARKET_TYPE.FUND_CN, FREQUENCE.ONE_MIN): 'index_min',
    (MARKET_TYPE.FUND_CN, FREQUENCE.FIVE_MIN): 'index_min',
    (MARKET_TYPE.FUND_CN, FREQUENCE.FIFTEEN_MIN): 'index_min',
    (MARKET_TYPE.FUND_CN, FREQUENCE.THIRTY_MIN): 'index_min',
    (MARKET_TYPE.FUND_CN, FREQUENCE.SIXTY_MIN): 'index_min',
    (MARKET_TYPE.FUND_CN, FREQUENCE.HOUR): 'index_min',
    (MARKET_TYPE.FUND_CN, FREQUENCE.TICK): 'index_transaction',
    (MARKET_TYPE.FUTURE_CN, FREQUENCE.DAY): 'future_day',
    (MARKET_TYPE.FUTURE_CN, FREQUENCE.ONE_MIN): 'future_min',
    (MARKET_TYPE.FUTURE_CN, FREQUENCE.FIVE_MIN): 'future_min',
    (MARKET_TYPE.FUTURE_CN, FREQUENCE.FIFTEEN_MIN): 'future_min',
    (MARKET_TYPE.FUTURE_CN, FREQUENCE.THIRTY_MIN): 'future_min',
    (MARKET_TYPE.FUTURE_CN, FREQUENCE.SIXTY_MIN): 'future_min',
    (MARKET_TYPE.FUTURE_CN, FREQUENCE.HOUR): 'future_min',
    (MARKET_TYPE.FUTURE_CN, FREQUENCE.TICK): 'future_transaction'
}



import os
import subprocess

USERPATH = os.path.expanduser('~')

subprocess.run(['killall','-9','streamlit'])

subprocess.run(args=["streamlit","run",f"{USERPATH}/rrshare/rrshare/rqWeb/stock_RS_OH_MA_to_streamlit.py"])






"""
ts_code	str	Y	TS代码
ann_date	str	Y	公告日期
end_date	str	Y	报告期
eps	float	Y	基本每股收益
dt_eps	float	Y	稀释每股收益
total_revenue_ps	float	Y	每股营业总收入
revenue_ps	float	Y	每股营业收入
capital_rese_ps	float	Y	每股资本公积
surplus_rese_ps	float	Y	每股盈余公积
undist_profit_ps	float	Y	每股未分配利润
extra_item	float	Y	非经常性损益
profit_dedt	float	Y	扣除非经常性损益后的净利润
gross_margin	float	Y	毛利
current_ratio	float	Y	流动比率
quick_ratio	float	Y	速动比率
cash_ratio	float	Y	保守速动比率
invturn_days	float	N	存货周转天数
arturn_days	float	N	应收账款周转天数
inv_turn	float	N	存货周转率
ar_turn	float	Y	应收账款周转率
ca_turn	float	Y	流动资产周转率
fa_turn	float	Y	固定资产周转率
assets_turn	float	Y	总资产周转率
op_income	float	Y	经营活动净收益
valuechange_income	float	N	价值变动净收益
interst_income	float	N	利息费用
daa	float	N	折旧与摊销
ebit	float	Y	息税前利润
ebitda	float	Y	息税折旧摊销前利润
fcff	float	Y	企业自由现金流量
fcfe	float	Y	股权自由现金流量
current_exint	float	Y	无息流动负债
noncurrent_exint	float	Y	无息非流动负债
interestdebt	float	Y	带息债务
netdebt	float	Y	净债务
tangible_asset	float	Y	有形资产
working_capital	float	Y	营运资金
networking_capital	float	Y	营运流动资本
invest_capital	float	Y	全部投入资本
retained_earnings	float	Y	留存收益
diluted2_eps	float	Y	期末摊薄每股收益
bps	float	Y	每股净资产
ocfps	float	Y	每股经营活动产生的现金流量净额
retainedps	float	Y	每股留存收益
cfps	float	Y	每股现金流量净额
ebit_ps	float	Y	每股息税前利润
fcff_ps	float	Y	每股企业自由现金流量
fcfe_ps	float	Y	每股股东自由现金流量
netprofit_margin	float	Y	销售净利率
grossprofit_margin	float	Y	销售毛利率
cogs_of_sales	float	Y	销售成本率
expense_of_sales	float	Y	销售期间费用率
profit_to_gr	float	Y	净利润/营业总收入
saleexp_to_gr	float	Y	销售费用/营业总收入
adminexp_of_gr	float	Y	管理费用/营业总收入
finaexp_of_gr	float	Y	财务费用/营业总收入
impai_ttm	float	Y	资产减值损失/营业总收入
gc_of_gr	float	Y	营业总成本/营业总收入
op_of_gr	float	Y	营业利润/营业总收入
ebit_of_gr	float	Y	息税前利润/营业总收入
roe	float	Y	净资产收益率
roe_waa	float	Y	加权平均净资产收益率
roe_dt	float	Y	净资产收益率(扣除非经常损益)
roa	float	Y	总资产报酬率
npta	float	Y	总资产净利润
roic	float	Y	投入资本回报率
roe_yearly	float	Y	年化净资产收益率
roa2_yearly	float	Y	年化总资产报酬率
roe_avg	float	N	平均净资产收益率(增发条件)
opincome_of_ebt	float	N	经营活动净收益/利润总额
investincome_of_ebt	float	N	价值变动净收益/利润总额
n_op_profit_of_ebt	float	N	营业外收支净额/利润总额
tax_to_ebt	float	N	所得税/利润总额
dtprofit_to_profit	float	N	扣除非经常损益后的净利润/净利润
salescash_to_or	float	N	销售商品提供劳务收到的现金/营业收入
ocf_to_or	float	N	经营活动产生的现金流量净额/营业收入
ocf_to_opincome	float	N	经营活动产生的现金流量净额/经营活动净收益
capitalized_to_da	float	N	资本支出/折旧和摊销
debt_to_assets	float	Y	资产负债率
assets_to_eqt	float	Y	权益乘数
dp_assets_to_eqt	float	Y	权益乘数(杜邦分析)
ca_to_assets	float	Y	流动资产/总资产
nca_to_assets	float	Y	非流动资产/总资产
tbassets_to_totalassets	float	Y	有形资产/总资产
int_to_talcap	float	Y	带息债务/全部投入资本
eqt_to_talcapital	float	Y	归属于母公司的股东权益/全部投入资本
currentdebt_to_debt	float	Y	流动负债/负债合计
longdeb_to_debt	float	Y	非流动负债/负债合计
ocf_to_shortdebt	float	Y	经营活动产生的现金流量净额/流动负债
debt_to_eqt	float	Y	产权比率
eqt_to_debt	float	Y	归属于母公司的股东权益/负债合计
eqt_to_interestdebt	float	Y	归属于母公司的股东权益/带息债务
tangibleasset_to_debt	float	Y	有形资产/负债合计
tangasset_to_intdebt	float	Y	有形资产/带息债务
tangibleasset_to_netdebt	float	Y	有形资产/净债务
ocf_to_debt	float	Y	经营活动产生的现金流量净额/负债合计
ocf_to_interestdebt	float	N	经营活动产生的现金流量净额/带息债务
ocf_to_netdebt	float	N	经营活动产生的现金流量净额/净债务
ebit_to_interest	float	N	已获利息倍数(EBIT/利息费用)
longdebt_to_workingcapital	float	N	长期债务与营运资金比率
ebitda_to_debt	float	N	息税折旧摊销前利润/负债合计
turn_days	float	Y	营业周期
roa_yearly	float	Y	年化总资产净利率
roa_dp	float	Y	总资产净利率(杜邦分析)
fixed_assets	float	Y	固定资产合计
profit_prefin_exp	float	N	扣除财务费用前营业利润
non_op_profit	float	N	非营业利润
op_to_ebt	float	N	营业利润／利润总额
nop_to_ebt	float	N	非营业利润／利润总额
ocf_to_profit	float	N	经营活动产生的现金流量净额／营业利润
cash_to_liqdebt	float	N	货币资金／流动负债
cash_to_liqdebt_withinterest	float	N	货币资金／带息流动负债
op_to_liqdebt	float	N	营业利润／流动负债
op_to_debt	float	N	营业利润／负债合计
roic_yearly	float	N	年化投入资本回报率
total_fa_trun	float	N	固定资产合计周转率
profit_to_op	float	Y	利润总额／营业收入
q_opincome	float	N	经营活动单季度净收益
q_investincome	float	N	价值变动单季度净收益
q_dtprofit	float	N	扣除非经常损益后的单季度净利润
q_eps	float	N	每股收益(单季度)
q_netprofit_margin	float	N	销售净利率(单季度)
q_gsprofit_margin	float	N	销售毛利率(单季度)
q_exp_to_sales	float	N	销售期间费用率(单季度)
q_profit_to_gr	float	N	净利润／营业总收入(单季度)
q_saleexp_to_gr	float	Y	销售费用／营业总收入 (单季度)
q_adminexp_to_gr	float	N	管理费用／营业总收入 (单季度)
q_finaexp_to_gr	float	N	财务费用／营业总收入 (单季度)
q_impair_to_gr_ttm	float	N	资产减值损失／营业总收入(单季度)
q_gc_to_gr	float	Y	营业总成本／营业总收入 (单季度)
q_op_to_gr	float	N	营业利润／营业总收入(单季度)
q_roe	float	Y	净资产收益率(单季度)
q_dt_roe	float	Y	净资产单季度收益率(扣除非经常损益)
q_npta	float	Y	总资产净利润(单季度)
q_opincome_to_ebt	float	N	经营活动净收益／利润总额(单季度)
q_investincome_to_ebt	float	N	价值变动净收益／利润总额(单季度)
q_dtprofit_to_profit	float	N	扣除非经常损益后的净利润／净利润(单季度)
q_salescash_to_or	float	N	销售商品提供劳务收到的现金／营业收入(单季度)
q_ocf_to_sales	float	Y	经营活动产生的现金流量净额／营业收入(单季度)
q_ocf_to_or	float	N	经营活动产生的现金流量净额／经营活动净收益(单季度)
basic_eps_yoy	float	Y	基本每股收益同比增长率(%)
dt_eps_yoy	float	Y	稀释每股收益同比增长率(%)
cfps_yoy	float	Y	每股经营活动产生的现金流量净额同比增长率(%)
op_yoy	float	Y	营业利润同比增长率(%)
ebt_yoy	float	Y	利润总额同比增长率(%)
netprofit_yoy	float	Y	归属母公司股东的净利润同比增长率(%)
dt_netprofit_yoy	float	Y	归属母公司股东的净利润-扣除非经常损益同比增长率(%)
ocf_yoy	float	Y	经营活动产生的现金流量净额同比增长率(%)
roe_yoy	float	Y	净资产收益率(摊薄)同比增长率(%)
bps_yoy	float	Y	每股净资产相对年初增长率(%)
assets_yoy	float	Y	资产总计相对年初增长率(%)
eqt_yoy	float	Y	归属母公司的股东权益相对年初增长率(%)
tr_yoy	float	Y	营业总收入同比增长率(%)
or_yoy	float	Y	营业收入同比增长率(%)
q_gr_yoy	float	N	营业总收入同比增长率(%)(单季度)
q_gr_qoq	float	N	营业总收入环比增长率(%)(单季度)
q_sales_yoy	float	Y	营业收入同比增长率(%)(单季度)
q_sales_qoq	float	N	营业收入环比增长率(%)(单季度)
q_op_yoy	float	N	营业利润同比增长率(%)(单季度)
q_op_qoq	float	Y	营业利润环比增长率(%)(单季度)
q_profit_yoy	float	N	净利润同比增长率(%)(单季度)
q_profit_qoq	float	N	净利润环比增长率(%)(单季度)
q_netprofit_yoy	float	N	归属母公司股东的净利润同比增长率(%)(单季度)
q_netprofit_qoq	float	N	归属母公司股东的净利润环比增长率(%)(单季度)
equity_yoy	float	Y	净资产同比增长率
rd_exp	float	N	研发费用
update_flag	str	N	更新标识

"""

import pandas as pd

from rrshare.rqFetch import pro


def fetch_fina_indicator(ts_code='', report_date='20201231'):
    """ report_date: four Q=  0331, 0630,0930,1231, genertor : report_date_list lastdate
        ts_code = '600519'--> all report_date
        report_date='20210331' --> all stock fina_factor
    """
    df = pro.fina_indicator_vip(ts_code=ts_code, period=report_date)
    print(list(df.columns))
    cols = ['ts_code', 'ann_date', 'end_date', 'eps', 'bps',
            'roe',
    #'profit_dedt', 'gross_margin','current_ratio', 'fcff', 'diluted2_eps', 
            #' 'ocfps', 'fcff_ps','netprofit_margin', 'grossprofit_margin', 'roe_waa', 'roe_dt', 'roa', 'roic'
            # 'dtprofit_to_profit', 'debt_to_assets',
              'or_yoy', 'op_yoy', 'dt_netprofit_yoy','cfps_yoy', 'dt_eps_yoy',
            #'ebt_yoy', 'netprofit_yoy', 'dt_netprofit_yoy', 'ocf_yoy', 'roe_yoy', 'bps_yoy', 'assets_yoy'
            ]
    df =df[cols].drop_duplicates()
    print(df)
    return df




if  __name__ == '__main__':
    ts_code = '300146.SZ, 002236.SZ,300674.SZ,600196.SH,002241.SZ,002415.SZ'
    fetch_fina_indicator(ts_code='', report_date='20210331')
    fetch_fina_indicator(ts_code='002236.SZ')


#!/usr/bin/python
# conding:utf-8

import time
import logging
from typing import List, Tuple, Union
import numpy as np
import pandas as pd
import pymysql
from sqlalchemy import create_engine

from rrshare.rqUtil.config_setting import setting
from rrshare.rqUtil.rqDate_trade import (rq_util_get_last_tradedate, rq_util_get_pre_trade_date)

logging.basicConfig(level=logging.DEBUG,format=' %(asctime)s-%(levelname)s-%(message)s ')

db_mysql ='rrshare'
user_mysql = 'root'
password_mysql = setting['MYSQL_PASSWORD']

logging.info(user_mysql)
logging.debug(password_mysql)


def conn_mysql(host="127.0.0.1", user=user_mysql,passwd=password_mysql,port=3306):
    return pymysql.connect(host=host, user=user,passwd=passwd,port=port,charset='utf8')

def conn_mysqldb(db='rrshare',host="127.0.0.1", user=user_mysql,passwd=password_mysql,port=3306):
    return pymysql.connect(host=host, user=user,passwd=passwd,database=db,port=port,charset='utf8')

def mysql_engine(db='rrshare',host="127.0.0.1",user=user_mysql,passwd=password_mysql,port=3306):
    try:
        return create_engine(f'mysql+pymysql://{user}:{passwd}.@{host}:{port}/{db}',encoding='utf8')
    except Exception as e:
        print(e)


class MysqlClass(object):
    
    def __init__(self, host="127.0.0.1", user=user_mysql, passwd=password_mysql):
        self.host = host
        self.user = user
        self.passwd = passwd
        self.port = 3306
        self.charset = 'utf8mb4'
        self.conn_mysql =  pymysql.connect(host=self.host,user=self.user,passwd=self.passwd,port=self.port,charset=self.charset)

    def create_mysqlDB(self, db):
        try:
            cursor = self.conn_mysql.cursor()
            cursor.execute(f"CREATE DATABASE IF NOT EXISTS {db} ")
            print(f'create database {db} ok!')
        except Exception as e:
            print(e)

    def conn_mysqldb(self, db):
        return pymysql.connect(host=self.host,
                               user=self.user, 
                               passwd=self.passwd,
                               db = db,
                               charset=self.charset)
        
    def conn_engine(self, db):
        return create_engine(f'mysql+pymysql://{self.user}:{self.passwd}@{self.host}:{self.port}/{db}',encoding='utf8')


    def create_mysqlDB_table(self, sql, db, table):
        conn =  self.conn_mysqldb(db)
        cursor = conn.cursor()
        cursor.execute(f"DROP TABLE IF EXISTS {table}")
        cursor.execute(sql)
        print(f'create table "{table}" in datadase "{db}"')
        cursor.close()
        conn.close()


    def write_to_mysql(self,df,db,table,if_exists='append'):
        #与MySQL建立连接，
        conn = create_engine(f'mysql+pymysql://{self.user}:{self.passwd}@{self.host}:{self.port}/{db}',encoding='utf8')
        #写入数据，table为表名，‘replace’表示如果同名表存在就替换掉
        try:
            pd.io.sql.to_sql(df,f"{table}", conn, if_exists=if_exists)
            print(f'写入数据库{db}表{table}, ok')
        except Exception as e:
            print(e)


    def read_mysql_table(self, db, table,N):
        #与MySQL建立连接，
        conn = create_engine(f'mysql+pymysql://{self.user}:{self.passwd}@{self.host}:{self.port}/{db}',encoding='utf8')
        startTD = rq_util_get_pre_trade_date(rq_util_get_last_tradedate(),N)
        startTD = f'"{startTD}"'   ### very important !!!
        #print(startTD)
        try:
            print(f'try to read data from {db}.{table} ...... ')
            sql = f"SELECT * FROM {table} WHERE time >= {startTD}"
            df = pd.read_sql(sql,conn)
            logging.info(f'read from database:{db} table:{table} N_tradedate \n {df.head()}')
            return df
        except Exception as e:
            print(e)

    def read__mysql_select(self,secs, db, table,N):
        #与MySQL建立连接，
        conn = create_engine(f'mysql+pymysql://{self.user}:{self.passwd}.@{self.host}:{self.port}/{db}',encoding='utf8')
        startTD = rq_util_get_pre_trade_date(rq_util_get_last_tradedate(),N)
        startTD = f'"{startTD}"'   ### very important !!!
        secs = ','.join(f"'{x}'" for x in secs) 
        #print(secs)
        try:
            print(f'try to read data from {db}.{table} ...... ')
            sql = f"SELECT * FROM {table} WHERE time >= {startTD} AND code IN({secs})"
            df = pd.read_sql(sql,conn)
            logging.info(f'read from database:{db} table:{table} N_tradedate \n {df.head()}')
            return df
        except Exception as e:
            print(e)


if __name__ == '__main__':

    mysqldb = MysqlClass()
    #mysqldb.create_mysqlDB('test_rrshare')
    mysqldb.read_mysql_table(db='test_rrshare', table='stock_day', N=2)
    #write_to_mysql(df, 'test_rrshare','test','append')
    #print(read_mysql_table('gm','stock_day',19))
    pass

    
    
    



from apscheduler.schedulers.blocking import BlockingScheduler

from rrshare.rqUtil import (rq_util_date_today,
                            is_trade_time_secs_cn,
                            rq_util_if_tradetime, 
                            rq_util_if_trade, 
                            rq_util_datetime_to_strdate)

from rrshare.rqFactor.stock_RS_OH_MA import update_stock_PRS_new

# 创建调度器：BlockingScheduler
scheduler = BlockingScheduler()
today_ = rq_util_datetime_to_strdate(rq_util_date_today())

def job_update_stock_PRS_new():
    try:
        update_stock_PRS_new()
    except Exception as e:
        print(e)
        
scheduler.add_job(job_update_stock_PRS_new, 'interval', minutes=1, id='job_stock_PRS_new') # next_run_time=datetime.datetime.now(),

con = rq_util_if_trade(today_) and is_trade_time_secs_cn()  # tradedate and tradetime 9:25 start run calulate PRS
print('start con: ', con)

while con:
    scheduler.start()




# coding:utf-8
import pymongo
from motor.motor_asyncio import AsyncIOMotorClient
from motor import MotorClient
import asyncio


def rq_util_sql_mongo_setting(uri='mongodb://localhost:27017/rrshare'):
    """
    explanation:
        根据给定的uri返回一个MongoClient实例，采用@几何建议以使用加密
    params:
        * uri ->:
            meaning: mongodb连接uri
            type: str
            optional: [null]
    return:
        MongoClient
    demonstrate:
        Not described
    output:
        Not described
    """

    # 采用@几何的建议,使用uri代替ip,port的连接方式
    # 这样可以对mongodb进行加密:
    # uri=mongodb://user:passwor@ip:port
    client = pymongo.MongoClient(uri)
    return client

# async


def rq_util_sql_async_mongo_setting(uri='mongodb://localhost:27017/rrshare'):
    """
    explanation:
        根据给定的uri返回一个异步AsyncIOMotorClient实例
    params:
        * uri ->:
            meaning: mongodb连接uri
            type: str
            optional: [null]
    return:
        AsyncIOMotorClient
    demonstrate:
        Not described
    output:
        Not described
    """
    # loop = asyncio.new_event_loop()
    # asyncio.set_event_loop(loop)
    try:
        loop = asyncio.get_event_loop()
    except RuntimeError:
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)

    # async def client():
    return AsyncIOMotorClient(uri, io_loop=loop)
    # yield  client()


ASCENDING = pymongo.ASCENDING
DESCENDING = pymongo.DESCENDING
rq_util_sql_mongo_sort_ASCENDING = pymongo.ASCENDING
rq_util_sql_mongo_sort_DESCENDING = pymongo.DESCENDING

if __name__ == '__main__':
    # test async_mongo
    client = rq_util_sql_async_mongo_setting().rrshare.stock_day
    print(client)



# pro
from rrshare.rqFetch.rqTusharepro import pro

# rqFetch

from rrshare.rqFetch.rqCodeName import (swl_index_to_name,stock_code_to_name,stock_code_belong_swl_name)
from rrshare.rqFetch.fetch_swl_daily import fetch_swl_daily_tspro_adv
from rrshare.rqFetch.fetch_basic_tusharepro import (fetch_delist_stock,
                                                        fetch_get_stock_list,
                                                        fetch_get_stock_list_adj)
from rrshare.rqFetch.rqFetchSnapshot_eq import (
        fetch_realtime_price_all, 
        fetch_realtime_price_stock)

from rrshare.rqFetch.fetch_stock_day_adj_fillna_from_tusharepro import (
                fetch_stock_day_adj_fillna_from_tusharepro,
                fetch_realtime_price_stock_day_adj)

from rrshare.rqFetch.fetch_stock_bar_hfq_from_tusharepro import (fetch_get_stock_bar_hfq_one,fetch_stock_list_tusharepro)

from rrshare.rqFetch.fetch_swl1_index_class_and_realtime import Swsindex



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import dash
import dash_core_components as dcc
import dash_html_components as html

from dash.dependencies import Input, Output

from rrshare.rqFetch import pro, jq
from rrshare.rqUtil import rq_util_get_last_tradedate, rq_util_log_info
from rrshare.rqFetch import stock_code_to_name


app = dash.Dash()

my_select_secs = ['600196.XSHG','300674.XSHE','300146.XSHE','002236.XSHE']
#codes = list(map(lambda x: x[0:6], my_select_secs))
options = []
for s in my_select_secs:
    sels = {}
    sels['label'] = stock_code_to_name(s[0:6])[0]
    sels['value'] = s
    options.append(sels)
rq_util_log_info(options)


app.layout = html.Div([
    html.H1('stock bar'),
    dcc.Dropdown(
    id = 'my_dropdown',
    options = options,
    value = my_select_secs[0]
    ),
    dcc.Graph(id='my-graph')
    ], style={'width': '500'})

@app.callback(Output('my-graph','figure'),[Input('my_dropdown','value')])
def update_graph(selected_dropdrown_value):

    #df = pro.daily(ts_code=selected_dropdrown_value, start_date='20210301', end_date=rq_util_get_last_tradedate().replace('-',''))
    df = jq.get_price(selected_dropdrown_value, count=250, end_date=rq_util_get_last_tradedate())
    #rq_util_log_info(f'\n {df}')

    return {
        'data':[
        {
            'x':df.index,
            'y':df.close, 
            #'y':df.low,
        }]}

app.css.append_css({'external_url': 'https://codepen.io/chriddyp/pen/bWLwgP.css'})

def main():
    app.run_server(host='0.0.0.0',port=8503, debug=True)

if __name__ == '__main__':
    main()
    
    
  



#!/usr/bin/python
import smtplib
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.header import Header


# OK !!!
#sender_qq为发件人的qq号码
sender_qq = 'romepeng'
#pwd为qq邮箱的授权码
pwd = 'fcthsbywwwyjbhjd'
#收件人邮箱receiver
receivers = ['rbobowang@hotmail.com','romepeng@outlook.com']  
#msg['To'] = ','.join(receiver) # 这里必须要把多个邮箱按照逗号拼接为字符串
#邮件的正文内容
mail_content = '你好，我是来自QQ ，romepeng,现在在进行一项用python登录qq邮箱发邮件的测试'
#邮件标题
mail_title = 'qq_romepeng 的邮件'

def send_mail(sender_qq='',pwd='',receiver='',mail_title='',mail_content=''):
	# qq邮箱smtp服务器
	host_server = 'smtp.qq.com'
	sender_qq_mail = 'romepeng@qq.com' 
	
	#ssl登录
	smtp = smtplib.SMTP_SSL(host_server, 465)
	#set_debuglevel()是用来调试的。参数值为1表示开启调试模式，参数值为0关闭调试模式
	smtp.set_debuglevel(1)
	smtp.ehlo(host_server)
	smtp.login(sender_qq, pwd)

	msg = MIMEText(mail_content, "plain", 'utf-8')
	msg["Subject"] = Header(mail_title, 'utf-8')
	msg["From"] = sender_qq_mail
	msg["To"] = ''.join(receivers)

	try:
		smtp.sendmail(sender_qq_mail, receivers, msg.as_string())
		print('sendmail success !')
	except smtplib.SMTPException as e:
		print(e)
	finally:
		smtp.quit()

if  __name__ == '__main__':
	send_mail(sender_qq=sender_qq,pwd=pwd,receiver=receivers,mail_title=mail_title,mail_content=mail_content)
	#注意MIMEText函数中的第二个参数为“plain”时，发送的是text文本。如果为“html”，则能发送网页格式文本邮件。




# coding : utf-8
import time
import datetime
from rrshare.rqUtil.rqParameter import TRADE_TIMES


def is_trade_time_secs_cn(time_: str=time.ctime()) -> bool:
    #print(time_)
    #print(datetime.datetime.now().strftime('%H:%M:%S'))
    time_hour_min =  time_.split(" ")[3]
    #print(time_hour_min)
    #print(TRADE_TIMES().TRADE_TIME_AM[0])
    con1 = (time_hour_min >= TRADE_TIMES().TRADE_TIME_AM_pre[0]) and (time_hour_min <= TRADE_TIMES().TRADE_TIME_AM_pre[1])
    #print(con1)
    con2 = (time_hour_min >= TRADE_TIMES().TRADE_TIME_PM_pre[0]) and (time_hour_min <= TRADE_TIMES().TRADE_TIME_PM_pre[1])
    #print(con2)
    con = con1 or con2
    return  con

def is_before_tradetime_secs_cn(time_: str=time.ctime()) -> bool:
    #print(time_)
    time_hour_min =  time_.split(" ")[3]
    #print(time_hour_min)
    return (time_hour_min < TRADE_TIMES().TRADE_TIME_AM_pre[0])

#print(is_trade_time_secs_cn())

#print(is_before_tradetime_secs_cn())




    



"""输入参数

名称	类型	必选	描述
ts_code	str	Y	股票代码
ann_date	str	N	公告日期
start_date	str	N	公告开始日期
end_date	str	N	公告结束日期
period	str	N	报告期(每个季度最后一天的日期，比如20171231表示年报)
report_type	str	N	报告类型： 参考下表说明
comp_type	str	N	公司类型：1一般工商业 2银行 3保险 4证券

输出参数

名称	类型	默认显示	描述
ts_code	str	Y	TS代码
ann_date	str	Y	公告日期
f_ann_date	str	Y	实际公告日期
end_date	str	Y	报告期
report_type	str	Y	报告类型 1合并报表 2单季合并 3调整单季合并表 4调整合并报表 5调整前合并报表 6母公司报表 7母公司单季表 8 母公司调整单季表 9母公司调整表 10母公司调整前报表 11调整前合并报表 12母公司调整前报表
comp_type	str	Y	公司类型(1一般工商业2银行3保险4证券)
basic_eps	float	Y	基本每股收益
diluted_eps	float	Y	稀释每股收益
total_revenue	float	Y	营业总收入
revenue	float	Y	营业收入
int_income	float	Y	利息收入
prem_earned	float	Y	已赚保费
comm_income	float	Y	手续费及佣金收入
n_commis_income	float	Y	手续费及佣金净收入
n_oth_income	float	Y	其他经营净收益
n_oth_b_income	float	Y	加:其他业务净收益
prem_income	float	Y	保险业务收入
out_prem	float	Y	减:分出保费
une_prem_reser	float	Y	提取未到期责任准备金
reins_income	float	Y	其中:分保费收入
n_sec_tb_income	float	Y	代理买卖证券业务净收入
n_sec_uw_income	float	Y	证券承销业务净收入
n_asset_mg_income	float	Y	受托客户资产管理业务净收入
oth_b_income	float	Y	其他业务收入
fv_value_chg_gain	float	Y	加:公允价值变动净收益
invest_income	float	Y	加:投资净收益
ass_invest_income	float	Y	其中:对联营企业和合营企业的投资收益
forex_gain	float	Y	加:汇兑净收益
total_cogs	float	Y	营业总成本
oper_cost	float	Y	减:营业成本
int_exp	float	Y	减:利息支出
comm_exp	float	Y	减:手续费及佣金支出
biz_tax_surchg	float	Y	减:营业税金及附加
sell_exp	float	Y	减:销售费用
admin_exp	float	Y	减:管理费用
fin_exp	float	Y	减:财务费用
assets_impair_loss	float	Y	减:资产减值损失
prem_refund	float	Y	退保金
compens_payout	float	Y	赔付总支出
reser_insur_liab	float	Y	提取保险责任准备金
div_payt	float	Y	保户红利支出
reins_exp	float	Y	分保费用
oper_exp	float	Y	营业支出
compens_payout_refu	float	Y	减:摊回赔付支出
insur_reser_refu	float	Y	减:摊回保险责任准备金
reins_cost_refund	float	Y	减:摊回分保费用
other_bus_cost	float	Y	其他业务成本
operate_profit	float	Y	营业利润
non_oper_income	float	Y	加:营业外收入
non_oper_exp	float	Y	减:营业外支出
nca_disploss	float	Y	其中:减:非流动资产处置净损失
total_profit	float	Y	利润总额
income_tax	float	Y	所得税费用
n_income	float	Y	净利润(含少数股东损益)
n_income_attr_p	float	Y	净利润(不含少数股东损益)
minority_gain	float	Y	少数股东损益
oth_compr_income	float	Y	其他综合收益
t_compr_income	float	Y	综合收益总额
compr_inc_attr_p	float	Y	归属于母公司(或股东)的综合收益总额
compr_inc_attr_m_s	float	Y	归属于少数股东的综合收益总额
ebit	float	Y	息税前利润
ebitda	float	Y	息税折旧摊销前利润
insurance_exp	float	Y	保险业务支出
undist_profit	float	Y	年初未分配利润
distable_profit	float	Y	可分配利润
update_flag	str	N	更新标识，0未修改1更正过
主要报表类型说明

代码	类型	说明
1	合并报表	上市公司最新报表（默认）
2	单季合并	单一季度的合并报表
3	调整单季合并表	调整后的单季合并报表（如果有）
4	调整合并报表	本年度公布上年同期的财务报表数据，报告期为上年度
5	调整前合并报表	数据发生变更，将原数据进行保留，即调整前的原数据
6	母公司报表	该公司母公司的财务报表数据
7	母公司单季表	母公司的单季度表
8	母公司调整单季表	母公司调整后的单季表
9	母公司调整表	该公司母公司的本年度公布上年同期的财务报表数据
10	母公司调整前报表	母公司调整之前的原始财务报表数据
11	调整前合并报表	调整之前合并报表原数据
12	母公司调整前报表	母公司报表发生变更前保留的原数据

"""

from rrshare.rqFetch import pro

df = pro.income(ts_code='600519.SH', start_date='20180101', end_date='', \
    fields='ts_code,ann_date,f_ann_date,end_date,report_type,comp_type,basic_eps,diluted_eps')
    
#df = pro.income_vip(period='20201231',fields='ts_code,ann_date,f_ann_date,end_date,report_type,comp_type,basic_eps,diluted_eps')
print(df)


# streamlit app
""" streamlit.line_chart(data=None, width=0, height=0, use_container_width=True)
    streamlit.area_chart(data=None, width=0, height=0, use_container_width=True)
    streamlit.bar_chart(data=None, width=0, height=0, use_container_width=True)
    streamlit.pyplot(fig=None, clear_figure=True, **kwargs)
    streamlit.plotly_chart(figure_or_data, width=0, height=0, use_container_width=False, sharing='streamlit', **kwargs)
    streamlit.bokeh_chart(figure, use_container_width=False)
    streamlit.pydeck_chart(pydeck_obj=None)
    streamlit.deck_gl_chart(spec=None, **kwargs)
    streamlit.graphviz_chart(figure_or_dot, width=0, height=0)

    st.multiselect('Multiselect', [1,2,3])
    st.write('http://datacenter.eastmoney.com/securities/api/data/v1/get?callback=jQuery112305791870244468389_1618109280782&sortColumns=NOTICE_DATE%2CSECURITY_CODE&sortTypes=-1%2C-1&pageSize=50&pageNumber=1&reportName=RPT_PUBLIC_OP_NEWPREDICT&columns=ALL&token=894050c76af8597a853f5b408b759f5d&filter=(REPORT_DATE%3D%272021-06-30%27)')
    st.text_area('Area for textual entry')
    trade_date = st.date_input('Date input')
    st.write(trade_date)
    st.time_input('Time entry')
    st.file_uploader('File uploader')
    import streamlit.components.v1 as components
    #url_ths=f'http://data.10jqka.com.cn/financial/yjyg/'
    #url_ths =  "http://emweb.securities.eastmoney.com/PC_HSF10/CoreConception/Index?type=soft&code=SZ300146#"
    # embed streamlit docs in a streamlit app
    #components.iframe(url_ths,  width=1400, height=700)    
    Create a static component
If your goal in creating a Streamlit Component is solely to display HTML code or render a chart from a Python visualization library, 
Streamlit provides two methods that greatly simplify the process: components.html() and components.iframe().
"""
from enum import unique
import time
import datetime
import streamlit as st
import pandas as pd
import numpy as np
from streamlit.components.v1 import components

from rrshare.rqUtil import (client_pgsql,read_data_from_pg,read_sql_from_pg)
from rrshare.rqUtil import  (rq_util_date_today,rq_util_get_pre_trade_date,rq_util_get_last_tradedate, trade_date_sse)
from rrshare.rqUtil import is_before_tradetime_secs_cn, is_trade_time_secs_cn, rq_util_if_trade


conn = client_pgsql('rrfactor')
lastTD = rq_util_get_last_tradedate()
today_ = rq_util_date_today().strftime('%Y-%m-%d') 
trade_date = today_  if today_ in trade_date_sse else rq_util_get_last_tradedate()
#now_ = datetime.datetime.now()

current = datetime.datetime.now()
year, month, day = current.year, current.month, current.day
start = datetime.datetime(year, month, day, 9, 23, 0)
noon_start = datetime.datetime(year, month, day, 12, 58, 0)

morning_end = datetime.datetime(year, month, day, 11, 31, 0)
end = datetime.datetime(year, month, day, 15, 2, 5)

st.write('last reload',current, 'last tradedate:',lastTD)

L = ['L1','L2','L3']

#print(current)
#print(current > start)
#print(current > noon_start)

def out_df_items(df):
    #df = df.round(2)
    date, time = df['trade_date'].values[0], df['time'].values[0]
    #date = df['trade_date'].values[0]
    df.drop(columns=['trade_date', 'time'], inplace=True)
    cols = list(df.columns)
    cols.remove('adj_factor')
    #print(cols)
    #cols.insert(1, "cn_name")
    df1 = df[cols]
    #print(df1.head())
    return df1, date, time


def out_swl_item(df):
    date_time = df['trade_date'].values[0]
    df.drop(columns=['trade_date'], inplace=True)
    return df, date_time


def swl_rs_valuation_all(level):
    table_name = f'swl_rs_valuation_{level}'
    df = pd.read_sql_table(table_name, conn)
    data = out_swl_item(df)
    st.write('申万行业相对强度和估值_{level} 更新时间：',data[1])
    st.dataframe(data[0],width=1400, height=900)
    

def swl_rs_valuation():
    df = pd.read_sql_table('swl1_rs_L1',conn) # swl1_rs_reaktime_L1
    df.rename(columns={'pct_change':'pct_chg'}, inplace=True)
    df = df.round(2)
    data = out_swl_item(df)
    
    st.write('申万行业相对强度和估值-L1实时: ',data[1])#: {df.trade_date.unique()}')
    st.dataframe(data[0], width=1400, height=900)

    for level in L:
        swl_rs_valuation_all(level)
    pass


def write_stock_RS_OH_MA_new():
    #st.text('相对强度 ')
    #table_name='stock_RS_OH_MA' if  current < start else 'stock_RS_OH_MA_new'
    table_name = 'stock_RS_OH_MA_new' 
    #print(table_name)
    cols = ['code', 'cn_name', 'close', 'pct_chg',
            #'ma5', 'ma10', 'ma20', 'ma60', 'ma120', 'ma250',
            'rs_5', 'rs_10', 'rs_20', 'rs_60', 'rs_120', 'rs_250',
            #'rt5', 'rt10', 'rt20', 'rt60', 'rt120', 'rt250',
            'H', 'L', 'OH', 'OL', 'swl_L1', 'swl_L2', 'swl_L3']
    cols1 = ['code','cn_name', 'close','pct_chg','OH',"OL","swl_L3",'rs_5', 'rs_10','rs_250']
    cols2= ['code','cn_name', 'close','pct_chg','ma20','ma250','rs_5','rs_20','rs_250','OH',"OL","swl_L3"]
    try:
        st.text('申万行业相对强度')
        df = pd.read_sql_table(table_name,conn)
        df.rename(columns={'name':'cn_name'},inplace=True)
        df.drop_duplicates(subset=None,keep='first',inplace=True)
        data = out_df_items(df)
        
        df1 = data[0][cols]
        df2 = data[0][cols1]
        data_by_pctchg = df1.sort_values(by='pct_chg', ascending=False)
        st.write("更新时间: ",data[1],data[2])
        #st.write(data[0])
        df2 = df2.dropna(axis=0,how='any')
        df2 = df2.sort_values(by='rs_5')
        #st.write(df2)
       
        st.write('涨停', str(len(data[0][data[0].pct_chg > 9.90])),'只' , ',   ',\
                    '跌停', str(len(data[0][data[0].pct_chg < -9.90])),'只' )
        st.write(data_by_pctchg, width=1200, height=600)
        
        st.write('一年新高', str(len(df[df.OH >98])),'只' )
        st.write(df2[(df2.OH > 98)], width=1200, height=600)

        st.write('一年新低', str(len(df[df.OL <2])),'只' )
        st.write(df2[df2.OL < 2], width=1200, height=600) 
  
        code = st.text_input('Input stock code:','600519')
        df1 = data[0][cols2]
        st.write(data[1], data[2])
        df_code = df1[df1.code == code]
        st.write(df_code)
    except Exception as e:
        print(e)    

        
def stock_select_PRS(table_name='stock_select_PRS'):
    try:
        df = pd.read_sql_table(table_name,conn)
        #df.rename(columns={'name':'cn_name'},inplace=True)
        print(df)
        data = out_df_items(df)

        st.write('相对强度 top', str(len(df)), '只')
        st.write(data[1], data[2])
        st.dataframe(df, width=1200, height=900)
    except Exception as e:
        print(e)


def stock_fundmentals(): #TODO
    #df = pd.read_sql_table('ROE_CF_SR_INC', conn)
    code = st.text_input('Input stock code:','300146')
    #code =st.multiselect('selects:',[1,2,3])
    #st.write([code])
    #st.text('股票基本面指标')
    #st.write(df[df.code  == code].T)


def stock_infomation():
    st.write("wencai","http://www.iwencai.com/unifiedwap/home/index?qs=pc_~soniu~info~all~homepage~enter")
    st.write('concept','http://q.10jqka.com.cn/gn/')
    #x = st.slider('x')
    st.text('业绩预告')
    #url_ths=f'http://data.10jqka.com.cn/financial/yjyg/ajax/yjyg/date/2021-03-31/board/YJYZ/field/enddate/ajax/{x}/free/1/'
    url_ths=f'http://data.10jqka.com.cn/financial/yjyg/'#2021-03-31/board/YJYZ/field/enddate/{x}/free/1/'
    st.write( url_ths)
    
    code=  st.text_input('Enter stock code:')
    def change_code(code='600519'):
        #em_code = 'SH600519'
        em_code =  'SZ' + str(code) if code < "333333" else 'SH' + str(code)
        #st.text(em_code)
        return em_code
    em_code = change_code(code)
    url = f"http://emweb.securities.eastmoney.com/PC_HSF10/CoreConception/Index?type=soft&code={em_code}#"
    st.write(f'stock {em_code} info {url}')

    url_hsgt="https://emrnweb.eastmoney.com/hsgt/search?"
    st.write(f'hsgt {url_hsgt}')

    st.write('cfi','http://quote.cfi.cn/cfi_industrydetails.aspx?ctype=5day&dtype=zws')
    st.write('财联社', 'https://cls.cn')
    st.write('雪球','https://xueqiu.com')

def other_info():
    st.write('HK','http://q.10jqka.com.cn/hk/indexYs/')
    st.write('US','http://q.10jqka.com.cn/usa/indexDefer/')
    st.write('航运指数', 'https://www.sse.net.cn/index/singleIndex?indexType=ccfi')
    st.write('中国新造船指数', 'https://www.cnpi.org.cn/index.html')
    st.write('global index ','http://q.10jqka.com.cn/global/')
    st.write('bond','https://cn.investing.com/rates-bonds/u.s.-10-year-bond-yield')
    st.write('myselect','http://quote.eastmoney.com/zixuan/?from=home')
    st.write("金十数据", "https://www.jin10.com/")
    st.write("商品价格","http://top.100ppi.com/zdb/detail-day---1.html")
    st.write('investing','https://hk.investing.com') 
    st.write("rome's blog", "https://rome.tk")
    st.write('cgf_v2ray','https://gitlab.com/api/v4/projects/30347863/repository/files/data%2Fnew%2Fv2yay-update.txt/raw?ref=main&private_token=glpat-9D-AL5GiX2modB2_XQZA')
    st.write('cgw_clash','https://gitlab.com/api/v4/projects/30347863/repository/files/data%2Fnew%2Fclash-update.yaml/raw?ref=main&private_token=glpat-9D-AL5GiX2modB2_XQZA')
def main():
    selects = st.sidebar.selectbox(
    "Menu:",
    ("stock_PRS",
        "swl", 
        "stock_PRS_top",
    #    'stock_fundamental',
        'stock_infomation',
        'other_info'))
    
    if selects == 'swl':
        swl_rs_valuation()
    if selects == "stock_PRS_top":
        stock_select_PRS()
    if selects == 'stock_PRS':
        write_stock_RS_OH_MA_new()
    if selects == 'stock_infomation':
        stock_infomation()
    if selects == 'other_info':
        other_info()


if __name__ == "__main__":
    #write_stock_RS_OH_MA_new()
    main()




import configparser
import json
import os
from multiprocessing import Lock
from rrshare.RQSetting.rqLocalize import rq_path, setting_path
from rrshare.rqUtil.rqSql import (
    rq_util_sql_async_mongo_setting,
    rq_util_sql_mongo_setting
)

# rrquant有一个配置目录存放在 ~/.rrquant
# 如果配置目录不存在就创建，主要配置都保存在config.json里面
# 貌似yutian已经进行了，文件的创建步骤，他还会创建一个setting的dir
# 需要与yutian讨论具体配置文件的放置位置 author:Will 2018.5.19

DEFAULT_MONGO = os.getenv('MONGODB', 'localhost')
DEFAULT_DB_URI = 'mongodb://{}:27017'.format(DEFAULT_MONGO)
CONFIGFILE_PATH = '{}{}{}'.format(setting_path, os.sep, 'config.ini')
INFO_IP_FILE_PATH = '{}{}{}'.format(setting_path, os.sep, 'info_ip.json')
STOCK_IP_FILE_PATH = '{}{}{}'.format(setting_path, os.sep, 'stock_ip.json')
FUTURE_IP_FILE_PATH = '{}{}{}'.format(setting_path, os.sep, 'future_ip.json')


class rq_Setting():

    def __init__(self, uri=None):
        self.lock = Lock()

        self.mongo_uri = uri or self.get_mongo()
        self.username = None
        self.password = None

        # 加入配置文件地址

    def get_mongo(self):
        config = configparser.ConfigParser()
        if os.path.exists(CONFIGFILE_PATH):
            config.read(CONFIGFILE_PATH)

            try:
                res = config.get('MONGODB', 'uri')
            except:
                res = DEFAULT_DB_URI

        else:
            config = configparser.ConfigParser()
            config.add_section('MONGODB')
            config.set('MONGODB', 'uri', DEFAULT_DB_URI)
            f = open('{}{}{}'.format(setting_path, os.sep, 'config.ini'), 'w')
            config.write(f)
            res = DEFAULT_DB_URI

        return res

    def get_config(
            self,
            section='MONGODB',
            option='uri',
            default_value=DEFAULT_DB_URI
    ):
        """[summary]
        Keyword Arguments:
            section {str} -- [description] (default: {'MONGODB'})
            option {str} -- [description] (default: {'uri'})
            default_value {[type]} -- [description] (default: {DEFAULT_DB_URI})
        Returns:
            [type] -- [description]
        """

        try:
            config = configparser.ConfigParser()
            config.read(CONFIGFILE_PATH)
            return config.get(section, option)
        except:
            res = self.client.rrquant.usersetting.find_one(
                {'section': section})
            if res:
                return res.get(option, default_value)
            else:
                self.set_config(section, option, default_value)
                return default_value

    def set_config(
            self,
            section='MONGODB',
            option='uri',
            default_value=DEFAULT_DB_URI
    ):
        """[summary]
        Keyword Arguments:
            section {str} -- [description] (default: {'MONGODB'})
            option {str} -- [description] (default: {'uri'})
            default_value {[type]} -- [description] (default: {DEFAULT_DB_URI})
        Returns:
            [type] -- [description]
        """
        t = {'section': section, option: default_value}
        self.client.rrquant.usersetting.update(
            {'section': section}, {'$set': t}, upsert=True)

        # if os.path.exists(CONFIGFILE_PATH):
        #     config.read(CONFIGFILE_PATH)
        #     self.lock.release()
        #     return self.get_or_set_section(
        #         config,
        #         section,
        #         option,
        #         default_value,
        #         'set'
        #     )

        #     # 排除某些IP
        #     # self.get_or_set_section(config, 'IPLIST', 'exclude', [{'ip': '1.1.1.1', 'port': 7709}])

        # else:
        #     f = open(CONFIGFILE_PATH, 'w')
        #     config.add_section(section)
        #     config.set(section, option, default_value)

        #     config.write(f)
        #     f.close()
        #     self.lock.release()
        #     return default_value

    def get_or_set_section(
            self,
            config,
            section,
            option,
            DEFAULT_VALUE,
            method='get'
    ):
        """[summary]
        Arguments:
            config {[type]} -- [description]
            section {[type]} -- [description]
            option {[type]} -- [description]
            DEFAULT_VALUE {[type]} -- [description]
        Keyword Arguments:
            method {str} -- [description] (default: {'get'})
        Returns:
            [type] -- [description]
        """

        try:
            if isinstance(DEFAULT_VALUE, str):
                val = DEFAULT_VALUE
            else:
                val = json.dumps(DEFAULT_VALUE)
            if method == 'get':
                return self.get_config(section, option)
            else:
                self.set_config(section, option, val)
                return val

        except:
            self.set_config(section, option, val)
            return val

    def env_config(self):
        return os.environ.get("MONGOURI", None)

    @property
    def client(self):
        return rq_util_sql_mongo_setting(self.mongo_uri)

    @property
    def client_async(self):
        return rq_util_sql_async_mongo_setting(self.mongo_uri)

    def change(self, ip, port):
        self.ip = ip
        self.port = port
        global DATABASE
        DATABASE = self.client
        return self


rqSETTING = rq_Setting()
DATABASE = rqSETTING.client.rrshare
DATABASE_ASYNC = rqSETTING.client_async.rrshare

"""
    with open(FUTURE_IP_FILE_PATH, "w") as f:
        json.dump(future_ip_list, f)
"""


import pandas as pd
import random
from rrshare.rqUtil.rqDate_trade import trade_date_sse
import pandas as pd
import random
from rrshare.rqUtil.rqDate_trade import trade_date_sse
from rrshare.rqUtil.rqDate import rq_util_date_str2int
from rrshare.rqUtil.rqCode import rq_util_code_tostr

_sh_url = 'http://www.sse.com.cn/market/dealingdata/overview/margin/a/rzrqjygk{}.xls'
_sz_url = 'http://www.szse.cn/api/report/ShowReport.szse?SHOWTYPE=xlsx&CATALOGID=1837_xxpl&txtDate={}&tab2PAGENO=1&ENCODE=1&TABKEY=tab2&random={}'


def fetch_get_sh_margin(date):
    """return shanghai margin data
    Arguments:
        date {str YYYY-MM-DD} -- date format
    'code','name','leveraged_balance','leveraged_buyout','leveraged_payoff','margin_left','margin_sell','margin_repay',
    标的证券代码	标的证券简称	本日融资余额(元) leveraged_balance	
    本日融资买入额(元)leveraged_buyout	本日融资偿还额(元) leveraged_payoff
    本日融券余量margin_left	本日融券卖出量 margin_sell	本日融券偿还量 margin_repay
    Returns:
        pandas.DataFrame -- res for margin data
    """
    if date in trade_date_sse:
        data = pd.read_excel(_sh_url.format(rq_util_date_str2int
                                            (date)), 1).assign(date=date).assign(sse='sh')

        data.columns = ['code', 'name', 'leveraged_balance', 'leveraged_buyout',
                        'leveraged_payoff', 'margin_left', 'margin_sell', 'margin_repay', 'date', 'sse']
        data.code = data.code.apply(lambda x: str(x)[0:6])
        return data
    else:
        pass


def fetch_get_sz_margin(date):
    """return shenzhen margin data
    Arguments:
        date {str YYYY-MM-DD} -- date format
    证券代码   证券简称     融资买入额(元) leveraged_buyout       融资余额(元)leveraged_balance  融券卖出量(股/份) margin_sell
       融券余量(股/份)margin_left        融券余额(元)      融资融券余额(元)        date sse
    Returns:
        pandas.DataFrame -- res for margin data
    """

    if date in trade_date_sse:
        sz_url = _sz_url.format(date, random.random())
        data = pd.read_excel(sz_url).assign(date=date, sse='sz')
        data.columns = ['code', 'name', 'leveraged_buyout', 'leveraged_balance',
                        'margin_sell', 'margin_left', 'margin_balance', 'totalbalance', 'date', 'sse']
        data.code = data.code.apply(lambda x: ('00000'+str(x))[-6:])
        return data
    else:
        pass


def fetch_get_margin_all(date):
    return pd.concat([fetch_get_sh_margin(date), fetch_get_sz_margin(date)]).set_index('code')
# http://data.eastmoney.com/zjlx/002433.html


def fetch_zjlx(code=None):
    # 获取资金流向

    pass


if __name__ == "__main__":
    print(fetch_get_sz_margin('2018-01-25'))
    print(fetch_get_sh_margin('2018-01-25'))
from rrshare.rqUtil.rqDate import rq_util_date_str2int
from rrshare.rqUtil.rqCode import rq_util_code_tostr

_sh_url = 'http://www.sse.com.cn/market/dealingdata/overview/margin/a/rzrqjygk{}.xls'
_sz_url = 'http://www.szse.cn/api/report/ShowReport.szse?SHOWTYPE=xlsx&CATALOGID=1837_xxpl&txtDate={}&tab2PAGENO=1&ENCODE=1&TABKEY=tab2&random={}'


def fetch_get_sh_margin(date):
    """return shanghai margin data
    Arguments:
        date {str YYYY-MM-DD} -- date format
    'code','name','leveraged_balance','leveraged_buyout','leveraged_payoff','margin_left','margin_sell','margin_repay',
    标的证券代码	标的证券简称	本日融资余额(元) leveraged_balance	
    本日融资买入额(元)leveraged_buyout	本日融资偿还额(元) leveraged_payoff
    本日融券余量margin_left	本日融券卖出量 margin_sell	本日融券偿还量 margin_repay
    Returns:
        pandas.DataFrame -- res for margin data
    """
    if date in trade_date_sse:
        data = pd.read_excel(_sh_url.format(rq_util_date_str2int
                                            (date)), 1).assign(date=date).assign(sse='sh')

        data.columns = ['code', 'name', 'leveraged_balance', 'leveraged_buyout',
                        'leveraged_payoff', 'margin_left', 'margin_sell', 'margin_repay', 'date', 'sse']
        data.code = data.code.apply(lambda x: str(x)[0:6])
        return data
    else:
        pass


def fetch_get_sz_margin(date):
    """return shenzhen margin data
    Arguments:
        date {str YYYY-MM-DD} -- date format
    证券代码   证券简称     融资买入额(元) leveraged_buyout       融资余额(元)leveraged_balance  融券卖出量(股/份) margin_sell
       融券余量(股/份)margin_left        融券余额(元)      融资融券余额(元)        date sse
    Returns:
        pandas.DataFrame -- res for margin data
    """

    if date in trade_date_sse:
        sz_url = _sz_url.format(date, random.random())
        data = pd.read_excel(sz_url).assign(date=date, sse='sz')
        data.columns = ['code', 'name', 'leveraged_buyout', 'leveraged_balance',
                        'margin_sell', 'margin_left', 'margin_balance', 'totalbalance', 'date', 'sse']
        data.code = data.code.apply(lambda x: ('00000'+str(x))[-6:])
        return data
    else:
        pass


def fetch_get_margin_all(date):
    return pd.concat([fetch_get_sh_margin(date), fetch_get_sz_margin(date)]).set_index('code')
# http://data.eastmoney.com/zjlx/002433.html


def fetch_zjlx(code=None):
    # 获取资金流向

    pass


if __name__ == "__main__":
    print(fetch_get_sz_margin('2018-01-25'))
    print(fetch_get_sh_margin('2018-01-25'))


import pandas as pd

from rrshare.rqUtil import (rq_util_date_str2int, rq_util_get_trade_range, rq_util_get_last_tradedate)
from rrshare.rqFetch import  pro
from rrshare.rqUtil import setting
from rrshare.rqUtil import (rq_util_code_tosrccode,rq_util_code_tostr)
from rrshare.rqUtil import read_data_from_pg, client_pgsql


def fetch_swl_daily_tspro(trade_date=""):
    """ 无trade_date输入；
        如果end_date不给值，为最后交易日
        swl_daily 数据比stock_daily要晚，晚上10以后？？
    """
    trade_date = rq_util_date_str2int(trade_date) if trade_date else rq_util_date_str2int(rq_util_get_last_tradedate())
    print(trade_date) 
    swl_day = pro.sw_daily(trade_date=trade_date)
    swl_day['index'] = swl_day['ts_code'].map(lambda x: x.split(".")[0])
    return swl_day
    

def fetch_swl_daily_tspro_adv(trade_date=''):
    df_swl = read_data_from_pg(table_name='swl_list', client=client_pgsql('rrshare'))
    df_swl_day = fetch_swl_daily_tspro(trade_date)
    df_swl_day_tspro = df_swl.merge(df_swl_day, how='outer', on='index')
    df_swl_day_tspro.drop(columns=['industry_name','ts_code','name'], inplace=True)
    #df_swl_day_tspro.rename(columns={'ts_code':'index_code'}, inplace=True)
    df_swl_day_tspro['trade_date'] = pd.to_datetime(df_swl_day_tspro['trade_date'], format='%Y-%m-%d')
    #df_swl_day_tspro  = df_swl_day_tspro[['trade_date', 'index','name','level','ts_code','industry_name',\
    #      'open','low','high','close','change','pct_change','vol','amount','pe','pb']] 
    #print(df_swl_day_tspro)
    return df_swl_day_tspro
    

if __name__ == '__main__':
    
    #print(pro.sw_daily(trade_date="20210401"))
    #print(fetch_swl_daily_tspro(trade_date='20210506'))
    print(fetch_swl_daily_tspro_adv())
    
    pass

    




"""
输入参数

名称	类型	必选	描述
ts_code	str	N	股票代码(二选一)
ann_date	str	N	公告日期 (二选一)
start_date	str	N	公告开始日期
end_date	str	N	公告结束日期
period	str	N	报告期(每个季度最后一天的日期，比如20171231表示年报)
type	str	N	预告类型(预增/预减/扭亏/首亏/续亏/续盈/略增/略减)
输出参数

名称	类型	描述
ts_code	str	TS股票代码
ann_date	str	公告日期
end_date	str	报告期
type	str	业绩预告类型(预增/预减/扭亏/首亏/续亏/续盈/略增/略减)
p_change_min	float	预告净利润变动幅度下限（%）
p_change_max	float	预告净利润变动幅度上限（%）
net_profit_min	float	预告净利润下限（万元）
net_profit_max	float	预告净利润上限（万元）
last_parent_net	float	上年同期归属母公司净利润
first_ann_date	str	首次公告日
summary	str	业绩预告摘要
change_reason	str	业绩变动原因

"""

from rrshare.rqFetch import pro

df = pro.forecast(ann_date='20190131', fields='ts_code,ann_date,end_date,type,p_change_min,p_change_max,net_profit_min')

df2 = pro.forecast_vip(period='20210630',fields='ts_code,ann_date,end_date,type,p_change_min,p_change_max,net_profit_min')

df2 = df2.sort_values(by='p_change_max', ascending=False)
df2 = df2[df2['net_profit_min'] > 5000]
print(df2)


# -*- coding: utf-8 -*-

import click


@click.group()
@click.help_option("-h", "--help")
def cli():
    """
    输入 rrshare [COMMAND] --help 查看命令详情
    例如：rrshare update --help
    """
    pass


@cli.command()
def make_config_path():
    """ make dir setting config path 
        .rrsdk /setting/-->  add VALUES to config.json
    """
    from rrshare.RQSetting.rqLocalize import  make_dir_path
    make_dir_path()
    

@cli.command()
def record_data():
    """update all stock and swl data and save to pgsql """
    from rrshare.record_all_data import main_record
    main_record()


@cli.command()
def start_streamlit():
    """start streamlit run PRS.py """
    from rrshare.start_streamlit import main_web
    main_web()


@cli.command()
def realtime_prs():
    """ run Timer.py --> start realtime PRS"""
    from rrshare.Timer import main_timer
    main_timer()


@cli.command()
def ipython():
    """打开 ipython """
    import rrshare
    pass
    try:
        from IPython import embed
    except ImportError:
        click.echo("请安装ipython:pip install ipython")
    else:
        embed()


@cli.command()
def version():
    """获取版本信息"""
    from rrshare import __version__
    click.echo("rrrshare=={}".format(__version__))


if __name__== '__main__':
    cli()




import time

from rrshare.rqUtil import PgsqlClass
from rrshare.rqFetch import Swsindex

def save_swl1_realtime():
    swi_rt = Swsindex().get_swsindex_L1_realtime()
    print(swi_rt)
    PgsqlClass().insert_to_psql(swi_rt, 'rrshare','swl1_realtime',if_exists='replace')


if __name__ == '__main__':
    save_swl1_realtime()

    pass




# coding: utf-8
import datetime
import time

from apscheduler.schedulers.blocking import BlockingScheduler

from rrshare.rqFactor import update_swl_rs_valuation
#from rrshare.rqFactor.rq_ROE_CF_SR_INC import update_roe_cf_sr_inc
from rrshare.rqFactor.stock_RS_OH_MA import (update_stock_PRS_day,
                                             update_stock_PRS_new)
from rrshare.rqSU import (rq_save_stock_day_adj_fillna_pg,
                          #rq_save_stock_day_pg, 
                          rq_save_stock_list_pg,
                          rq_save_swl_industry_list_stock_belong,
                          rq_save_swl_day_pg)
from rrshare.rqUtil import (rq_util_date_today,
                            is_trade_time_secs_cn,
                            rq_util_if_tradetime, 
                            rq_util_if_trade, 
                            rq_util_datetime_to_strdate)

from rrshare.rqSU import save_swl1_realtime

from rrshare.rqFactor import swl1_index_realtime_prs

today_ = rq_util_datetime_to_strdate(datetime.date.today())


# 创建调度器：BlockingScheduler
scheduler = BlockingScheduler()
#scheduler = BackgroundScheduler()


def job_start_first_all():  # timer before start update first
    
    #update_roe_cf_sr_inc()

    rq_save_swl_industry_list_stock_belong()
    rq_save_swl_day_pg()

    rq_save_stock_list_pg()
    rq_save_stock_day_pg()
    rq_save_stock_day_adj_fillna_pg()

    update_swl_rs_valuation()

    update_stock_PRS_day()
    #update_stock_PRS()


def job_save_stock_day():
    rq_save_stock_list_pg()
    rq_save_stock_day_pg('2018-01-01')
    rq_save_stock_day_adj_fillna_pg('2018-01-01')


def job_save_swl_day():
    rq_save_swl_list_pg()
    rq_save_stock_belong_swl_pg()
    rq_save_swl_day_pg()


def job_update_stock_PRS_pre():
    update_stock_PRS_day()


def job_update_stock_PRS_new():
    update_stock_PRS_new()

def job_update_swl1_realtime_prs():
    save_swl1_realtime()
    swl1_index_realtime_prs()

def job_update_ROE_CF_SR_INC():
    #update_roe_cf_sr_inc()
    pass


def dojob():
    # job_start_first_all()
    # stock day
    scheduler.add_job(job_save_stock_day, 'cron', day_of_week='mon-sun',next_run_time=datetime.datetime.now(),
                      hour=19, minute=15, id='job_save_stock_day')
    # swl day
    scheduler.add_job(job_save_swl_day, 'cron', day_of_week='mon-fri',next_run_time=datetime.datetime.now(),
                      hour=21, minute=15, id='job_save_swl_day')
    # add stock PRS_pre
    scheduler.add_job(job_update_stock_PRS_pre, 'cron', 
                      day_of_week='mon-fri', hour=20, minute=30, next_run_time=datetime.datetime.now(),
                       id='job_stock_PRS_pre')
    # realtime PRS_new # run now() first
    scheduler.add_job(job_update_stock_PRS_new, 'interval', minutes=1, next_run_time=datetime.datetime.now(),
                      start_date='2021-04-05 9:28:00', end_date='2021-12-31 23:01:00', id='job_stock_PRS_new')

    # realtime swl1_prs
    scheduler.add_job(job_update_swl1_realtime_prs, 'interval', minutes=1, next_run_time=datetime.datetime.now(),
                      start_date='2021-05-05 9:26:00', end_date='2021-12-31 23:01:00', id='job_swl1_prs_realtime')


    # update_ROE_CF_SR_INC
    # scheduler.add_job(job_update_ROE_CF_SR_INC, 'cron', day_of_week='mon-sun',next_run_time=datetime.datetime.now(),
    #                  hour=8, minute=10, id='job_update_ROE_CF_SR_INC')

    scheduler.start()
    pass

#job_start_first_all()


def main_timer():
    scheduler.add_job(job_update_stock_PRS_new, 'interval', minutes=1, next_run_time=datetime.datetime.now(), id='job_stock_PRS_new')
    scheduler.add_job(job_update_swl1_realtime_prs,'interval', minutes=1, next_run_time=datetime.datetime.now(), id='job_swl1_prs_realtime')

    while  rq_util_if_trade(today_) and rq_util_if_tradetime():
        scheduler.start()

if __name__ == '__main__':
    main_timer()
    








# coding: utf-8
import pandas as pd
import numpy as np
import os
import time
import datetime
import pymongo
import tushare as ts
from sqlalchemy import create_engine
import psycopg2  
import warnings
warnings.filterwarnings("ignore")

from rrshare.rqUtil.rqLogs import (rq_util_log_debug, rq_util_log_expection,
                                     rq_util_log_info)
from rrshare.rqUtil.rqCode import (rq_util_code_tosrccode, rq_util_code_tostr)
from rrshare.rqUtil.rqDate_trade import rq_util_get_trade_range
from rrshare.rqUtil.rqDate import (rq_util_date_str2int,rq_util_date_int2str)

from rrshare.rqUtil import (PgsqlClass, rq_util_get_last_tradedate)
from rrshare.rqFetch import (pro, fetch_stock_day_adj_fillna_from_tusharepro)
from rrshare.rqFetch.fetch_swl_daily import fetch_swl_daily_tspro_adv

from rrshare.rqFetch import (fetch_stock_bar_hfq_from_tusharepro, fetch_stock_list_tusharepro)

lastTD = rq_util_get_last_tradedate()
print(lastTD)

def create_pgsql(database='rrshare'):
    try:
        PgsqlClass().create_psqlDB(db_name=database)
    except Exception as e:
        print(e)


"""
def rq_util_get_sql_file(path='/home/rome/rrshare/rrshare/sql', file_sql=''):
    file_name_sql = os.path.join(path, file_sql)
    rq_util_log_info(file_name_sql)
    with open(file_name_sql, 'r') as f:
        tsql = f.read()
    return tsql
"""

def create_pgsql_table(tabel_name='', tsql='', db_name='rrshare'):
    try:
        PgsqlClass().create_table(tabel_name, tsql)
    except Exception as e:
        print(e)

def client_pgsql(database='rrshare'):
    try:
        return PgsqlClass().client_pg(db_name=database)
    except Exception as e:
        print(e)

def save_data_to_postgresql(name_biao,data,if_exists='replace',client=client_pgsql()):
        data.to_sql(name_biao,client,index=False,if_exists=if_exists)
    
def load_data_from_postgresql(mes='',client=client_pgsql()):
    res=pd.read_sql(mes,client)
    return res

def read_data_from_pg(table_name='', client=client_pgsql()):
    res = pd.read_sql_table(table_name, client)
    return res

def download_day_data_from_tushare(trade_date='2020010102'):   
    trade_date = trade_date.replace('-', '') #兼容设置以防日期格式为2001-10-20格式
    lastEx = None
    retry = 10
    for _ in range(retry):
        try:
            df_daily=pro.query('daily',trade_date=trade_date)
            df_daily_basic=pro.query('daily_basic',trade_date=trade_date)
            df_factor=pro.query('adj_factor',trade_date=trade_date)             
            break 
        except Exception as ex:
            lastEx = ex
            rq_util_log_info("[{}]TuSharePro数据异常: {}, retrying...".format(trade_date, ex))
        else:
            rq_util_log_info("[{}]TuSharePro异常: {}, retried {} times".format(trade_date,lastEx, retry))
            return None   
    df=pd.merge(df_daily,df_factor,how='left')
    res=pd.merge(df,df_daily_basic,how='left').sort_values(by = 'ts_code')
    res['code']=res['ts_code'].apply(lambda x:x[:6]) #x[7:9].lower()
    res['trade_date'] = pd.to_datetime(res['trade_date'], format='%Y-%m-%d')
    #print(res)
    return res 

def fetch_stock_daily_adj_from_tusharepro(trade_date='20200101'):
    trade_date = trade_date.replace('-', '') #兼容设置以防日期格式为2001-10-20格式
    lastEx = None
    retry = 10
    for _ in range(retry):
        try:
            df_daily=pro.query('daily',trade_date=trade_date)
            #df_daily_basic=pro.query('daily_basic',trade_date=trade_date)
            df_factor=pro.query('adj_factor',trade_date=trade_date)             
            break 
        except Exception as ex:
            lastEx = ex
            rq_util_log_info("[{}]TuSharePro数据异常: {}, retrying...".format(trade_date, ex))
        else:
            rq_util_log_info("[{}]TuSharePro异常: {}, retried {} times".format(trade_date,lastEx, retry))
            return None   
    #df=pd.merge(df_daily,df_factor,how='left')
    res=pd.merge(df_factor,df_daily,how='left').sort_values(by = 'ts_code')
    res.fillna({'change':0,'pct_chg':0,'vol':0,'amount':0}, inplace=True)
    res['code']=res['ts_code'].apply(lambda x:x[:6]) #x[7:9].lower()
    res['trade_date'] = pd.to_datetime(res['trade_date'], format='%Y-%m-%d')
    #print(res)
    return res


def rq_fetch_stock_day_pg(code=None,start_date='20190101',end_date='20500118',data='*'):
    name_biao='stock_day'
    if isinstance(code,list):
        code="','".join(code)
        mes='select '+ data+' from '+name_biao+" where  trade_date >= date_trunc('day',timestamp '"+start_date+"') \
                and trade_date <= date_trunc('day',timestamp '"+end_date+"') and code in ('"+code+"')\
                order by trade_date ASC;"
        try:   
            t=time.time()        
            res=load_data_from_postgresql(mes=mes)
            t1=time.time()
            rq_util_log_info('load '+ name_biao+ ' success,take '+str(round(t1-t,2))+' S')              
        except Exception as e:
            print(e)
            res=None
    else:
        rq_util_log_info('code type is not list, please cheack it.')         
    return res

def rq_fetch_stock_list_pg(name_biao='stock_list'):
    mes='select * from '+name_biao+";"    
    try:   
        t=time.time()        
        res=load_data_from_postgresql(mes=mes)
        t1=time.time()
        rq_util_log_info('load '+ name_biao+ ' success,take '+str(round(t1-t,2))+' S')              
    except Exception as e:
        print(e)
        res=None
    return res
  

def rq_save_stock_list_pg():
    stock_list_l= pro.stock_basic(exchange_id='', is_hs='',list_status='L' , fields='ts_code,symbol,name,area,industry,fullname,enname,market,exchange,curr_type,list_status,list_date,delist_date,is_hs')  
    #stock_list_D= pro.stock_basic(exchange_id='', is_hs='',list_status='D' , fields='ts_code,symbol,name,area,industry,fullname,enname,market,exchange,curr_type,list_status,list_date,delist_date,is_hs')  
    #stock_list_P= pro.stock_basic(exchange_id='', is_hs='',list_status='P' , fields='ts_code,symbol,name,area,industry,fullname,enname,market,exchange,curr_type,list_status,list_date,delist_date,is_hs')          
    #stock_list=pd.concat([stock_list_l,stock_list_D],axis=0)
    #stock_list=pd.concat([stock_list_l,stock_list_P],axis=0)
    stock_list = stock_list_l
    stock_list['code']=stock_list['symbol']
    #print(df)
    try:   
        t=time.time()    
        save_data_to_postgresql('stock_list',stock_list)
        t1=time.time()
        rq_util_log_info('save stock_list data success,take '+str(round(t1-t,2))+' S') 
    except Exception as e:        
        print(e) 


def rq_save_stock_day_pg(start_date='20190104'):    
    t = time.localtime(time.time())
    if int(time.strftime('%H%M%S',t))<190000:   #晚上七点之后在更新当天数据，以免不及时
        t = time.localtime(time.time()-3600*24)
        tS = time.strftime("%Y-%m-%d", t)
    else:                
        tS = time.strftime("%Y-%m-%d", t)    
    end_date=tS
    print(end_date)
    try: 
        mes='select distinct trade_date FROM stock_day;'
        trade_data_pg = load_data_from_postgresql(mes).trade_date.tolist()
        print(trade_data_pg[-3:])
        for i in range(len(trade_data_pg)):
            trade_data_pg[i]=trade_data_pg[i].strftime("%Y-%m-%d")
    except: #第一次运行
        trade_data_pg=list()
    print(trade_data_pg[-3:])
    if isinstance(start_date,int):
        start_date=rq_util_date_int2str(start_date)
    elif len(start_date)==8:
        start_date=start_date[0:4]+'-'+start_date[4:6]+'-'+start_date[6:8]
    #print(start_date)
    trade_date=rq_util_get_trade_range(start_date,end_date) 
    #print(trade_date)
    trade_date2=list(set(trade_date).difference(set(trade_data_pg))) #差集
    trade_date2.sort()
    print(trade_date2)
    if len(trade_date2)==0:
        rq_util_log_info('Stock day is up to date and does not need to be updated')
    for i in trade_date2:
        print(i)
        try:
            t=time.time()        
            df=download_day_data_from_tushare(i)    
            #i=i[7:10].lower()+i[0:6]
            save_data_to_postgresql('stock_day',df,'append')
            t1=time.time()   
            rq_util_log_info('save '+i+' stock day success,take '+str(round(t1-t,2))+' S')        
        except Exception as e:
            print(e)


def rq_save_stock_day_adj_fillna_pg(start_date='20190101'): #20050101
    table_name='stock_day_adj_fillna'    
    t = time.localtime(time.time())
    if int(time.strftime('%H%M%S',t))<180000:   #晚上6点之后在更新当天数据，以免不及时
        t = time.localtime(time.time()-3600*24)
        tS = time.strftime("%Y-%m-%d", t)
    else:                
        tS = time.strftime("%Y-%m-%d", t)    
    end_date=tS
    print(end_date)
    try: 
        mes=f'select distinct trade_date FROM {table_name};'
        trade_data_pg = load_data_from_postgresql(mes).trade_date.tolist()
        #print(trade_data_pg[-3:])
        for i in range(len(trade_data_pg)):
            trade_data_pg[i]=trade_data_pg[i].strftime("%Y-%m-%d")
    except: #第一次运行
        trade_data_pg=list()
    #print(trade_data_pg[-3:])
    if isinstance(start_date,int):
        start_date=rq_util_date_int2str(start_date)
    elif len(start_date)==8:
        start_date=start_date[0:4]+'-'+start_date[4:6]+'-'+start_date[6:8]
    #print(start_date)
    trade_date=rq_util_get_trade_range(start_date,end_date) 
    #print(trade_date)
    trade_date2=list(set(trade_date).difference(set(trade_data_pg))) #差集 在trade_date 中 不在trade_data_pg
    trade_date2.sort()
    print(trade_date2)
    if len(trade_date2)==0:
        rq_util_log_info('Stock day adj is up to date and does not need to be updated')
    for i in trade_date2:
        print(i)
        try:
            t=time.time()        
            df=fetch_stock_day_adj_fillna_from_tusharepro(i)    
            #i=i[7:10].lower()+i[0:6]
            save_data_to_postgresql(table_name, df, 'append')
            t1=time.time()
            tt = round((t1-t),4)
            time.sleep(4)
            rq_util_log_info(f'save {i} {table_name} success,take {tt}S')        
        except Exception as e:
            print(e)




def rq_save_swl_day_pg(start_date='2020-01-01'): #from 2015-01-04
    table_name='swl_day'
    t = time.localtime(time.time())
    if int(time.strftime('%H%M%S',t))<210000:   #晚上9点之后在更新当天数据，以免不及时
        t = time.localtime(time.time()-3600*24)
        tS = time.strftime("%Y-%m-%d", t)
    else:
        tS = time.strftime("%Y-%m-%d", t)
    end_date=tS
    print(end_date)
    try:
        mes=f'select distinct trade_date FROM {table_name};'
        #mes='SELECT DISTINCT trade_date FROM swl_day;'
        trade_data_pg = load_data_from_postgresql(mes).trade_date.tolist()
        #print(trade_data_pg[-3:])
        for i in range(len(trade_data_pg)):
            trade_data_pg[i]=trade_data_pg[i].strftime("%Y-%m-%d")
            #trade_data_pg = list(map(lambda x: x.strftime('%Y-%m-%d'), trade_data_pg))
            #print(len(trade_data_pg))
            #print(trade_data_pg[-1:])
    except: #第一次运行
        trade_data_pg=list()
        #print(trade_data_pg[-3:])
    if isinstance(start_date,int):
        start_date=rq_util_date_int2str(start_date)
    elif len(start_date)==8:
        start_date=start_date[0:4]+'-'+start_date[4:6]+'-'+start_date[6:8]
        #print(start_date)
    trade_date=rq_util_get_trade_range(start_date,end_date)
    #print(trade_date)
    trade_date2=list(set(trade_date).difference(set(trade_data_pg))) #差集 在trade_date 中 不在trade_data_pg
    trade_date2.sort()
    print(trade_date2)

    if len(trade_date2)==0:
        rq_util_log_info('swl day is up to date and does not need to be updated')
    for i in trade_date2:
        print(i)
        try:
            t=time.time()
            df=fetch_swl_daily_tspro_adv(trade_date=i)
            save_data_to_postgresql(table_name, df, 'append')
            t1=time.time()
            tt = round((t1-t),4)
            time.sleep(1)
            rq_util_log_info(f'save {i} {table_name} success,take {tt}S')
        except Exception as e:
            print(e)


def rq_save_stock_bar_hfq_pg(start_date='19900101'): #20050101i#TODO
    table_name='stock_bar_hfq'    
    t = time.localtime(time.time())
    if int(time.strftime('%H%M%S',t))<180000:   #晚上6点之后在更新当天数据，以免不及时
        t = time.localtime(time.time()-3600*24)
        tS = time.strftime("%Y-%m-%d", t)
    else:                
        tS = time.strftime("%Y-%m-%d", t)    
    end_date=tS
    print(end_date)
    try: 
        mes=f'select distinct trade_date FROM {table_name};'
        trade_data_pg = load_data_from_postgresql(mes).trade_date.tolist()
        #print(trade_data_pg[-3:])
        for i in range(len(trade_data_pg)):
            trade_data_pg[i]=trade_data_pg[i].strftime("%Y-%m-%d")
    except: #第一次运行
        trade_data_pg=list()
    #print(trade_data_pg[-3:])
    if isinstance(start_date,int):
        start_date=rq_util_date_int2str(start_date)
    elif len(start_date)==8:
        start_date=start_date[0:4]+'-'+start_date[4:6]+'-'+start_date[6:8]
    #print(start_date)
    trade_date=rq_util_get_trade_range(start_date,end_date) 
    #print(trade_date)
    trade_date2=list(set(trade_date).difference(set(trade_data_pg))) #差集 在trade_date 中 不在trade_data_pg
    trade_date2.sort()
    print(trade_date2)
    if len(trade_date2)==0:
        rq_util_log_info('Stock day adj is up to date and does not need to be updated')
    for i in trade_date2:
        print(i)
        try:
            t=time.time()        
            df=fetch_stock_day_adj_fillna_from_tusharepro(i)    
            #i=i[7:10].lower()+i[0:6]
            save_data_to_postgresql(table_name, df, 'append')
            t1=time.time()
            tt = round((t1-t),4)
            time.sleep(4)
            rq_util_log_info(f'save {i} {table_name} success,take {tt}S')        
        except Exception as e:
            print(e)



if __name__ == '__main__':          
    
    rq_save_stock_list_pg()
    #rq_save_swl_day_pg('20200101')

    #储存日线数据  包含ts_pro中"daily" "daily_basic" "adj_factor"所有内容 
    #由于采用日期对比机制进行储存，可以增量储存之前数据
    #rq_save_stock_day_pg('20180101')  #rq_save_stock_day_pg('20180101')储存起始日期
    #rq_save_stock_day_adj_fillna_pg('20200101')
        
    pass



import json
import os , sys, platform,shutil, cmd

from rrshare.rqUtil.rqSingleton import Singleton, Singleton_wraps

# diffrence OS path
path = os.path.expanduser('~')
rq_path = '{}{}{}'.format(path, os.sep, '.rrshare')
#print(platform.system(),path, rq_path)


def get_path_file_name(path_name, file_name):
    # diffrence OS path
    if platform.system() == 'Linux':
        path_name_chg = ''.join([rq_path,'/' ,path_name, '/', file_name])
    if platform.system() == 'Windows':
        path_name_chg =  ''.join([rq_path,'\\', path_name, '\\',file_name])
    print(path_name_chg)
    return path_name_chg

path_setting = get_path_file_name('setting','config.json')


@Singleton_wraps
class Setting(object):
    def __init__(self, path_config=path_setting):
        self.path_config = path_config
        
    def setting(self):
        config = open(self.path_config)
        return json.load(config)

setting = Setting().setting()
#print(setting)

if __name__ == '__main__':
    s = Setting()
    #print(id(s), id(Setting()))
    print(s.setting()['TSPRO_TOKEN'])

    





# coding: utf-8
import pandas as pd
import numpy as np
import time
import datetime
import tushare as ts
import warnings
warnings.filterwarnings("ignore")

from rrshare.rqUtil.rqLogs import (rq_util_log_debug, rq_util_log_expection,
                                     rq_util_log_info)
from rrshare.rqUtil.rqCode import (rq_util_code_tosrccode, rq_util_code_tostr)
from rrshare.rqUtil.rqDate_trade import rq_util_get_trade_range
from rrshare.rqUtil.rqDate import (rq_util_date_str2int,rq_util_date_int2str)

from rrshare.rqUtil import setting, PERIODS

PERIODS = PERIODS().PERIODS
token= setting['TSPRO_TOKEN']

try:
    ts.set_token(token)
    pro=ts.pro_api()
except Exception as e:
    print(e)


def fetch_stock_bar_from_tusharepro(trade_date='20190102'):   
    trade_date = trade_date.replace('-', '') #兼容设置以防日期格式为2001-10-20格式
    lastEx = None
    retry = 5
    for _ in range(retry):
        try:
            df_daily=pro.query('daily',trade_date=trade_date)
            #print(df_daily)
            df_factor=pro.query('adj_factor',trade_date=trade_date)             
            #print(df_factor)
            break 
        except Exception as ex:
            lastEx = ex
            rq_util_log_info("[{}]TuSharePro数据异常: {}, retrying...".format(trade_date, ex))
    else:
        rq_util_log_info("[{}]TuSharePro异常: {}, retried {} times".format(trade_date,lastEx, retry))
        return None   
    df=pd.merge(df_daily,df_factor,how='left')
    df['code']=df['ts_code'].apply(lambda x:x[:6]) #x[7:9].lower()
    df['trade_date'] = pd.to_datetime(df['trade_date'], format='%Y-%m-%d')
    df['vwap'] = 10*df['amount'] / df['vol']
    for col in ['close','open','high','low','pre_close','vwap']:
        df[col] = df[col] * df['adj_factor']
        #df['pct_change'] = 100*(df['close']-df['pre_close'])/df['pre_close']
        df[col] = df[col].map(lambda x: "{:+.2f}".format(x)).astype(float)
    #print(df[df.code=='600519'])
    return df 


def fetch_stock_day_adj_fillna_from_tusharepro(trade_date='20050104'):
    trade_date = trade_date.replace('-', '') #兼容设置以防日期格式为2001-10-20格式
    lastEx = None
    retry = 10
    for _ in range(retry):
        try:
            df_daily=pro.query('daily',trade_date=trade_date)
            #df_daily_basic=pro.query('daily_basic',trade_date=trade_date)
            df_factor=pro.query('adj_factor',trade_date=trade_date)             
            break 
        except Exception as ex:
            lastEx = ex
            rq_util_log_info("[{}]TuSharePro数据异常: {}, retrying...".format(trade_date, ex))
        else:
            rq_util_log_info("[{}]TuSharePro异常: {}, retried {} times".format(trade_date,lastEx, retry))
            return None   
    #df=pd.merge(df_daily,df_factor,how='left')
    res=pd.merge(df_factor,df_daily,how='left').sort_values(by = 'ts_code')
    res.fillna({'change':0,'pct_chg':0,'vol':0,'amount':0}, inplace=True)
    res['code']=res['ts_code'].apply(lambda x:x[:6]) #x[7:9].lower()
    res['trade_date'] = pd.to_datetime(res['trade_date'], format='%Y-%m-%d')
    #print(res)
    return res


def fetch_stock_list_tusharepro():    
    stock_list_l= pro.stock_basic(exchange_id='', is_hs='',list_status='L' , fields='ts_code,symbol,name,area,industry,fullname,enname,market,exchange,curr_type,list_status,list_date,delist_date,is_hs')  
    stock_list_D= pro.stock_basic(exchange_id='', is_hs='',list_status='D' , fields='ts_code,symbol,name,area,industry,fullname,enname,market,exchange,curr_type,list_status,list_date,delist_date,is_hs')  
    stock_list_P= pro.stock_basic(exchange_id='', is_hs='',list_status='P' , fields='ts_code,symbol,name,area,industry,fullname,enname,market,exchange,curr_type,list_status,list_date,delist_date,is_hs')          
    stock_list=pd.concat([stock_list_l,stock_list_D],axis=0)
    stock_list=pd.concat([stock_list,stock_list_P],axis=0)
    return stock_list


def fetch_get_stock_bar_hfq_one(name, start='19900101', end='21001231', adj='hfq', freq='D', ma=PERIODS, adjfactor=True, type_='pd'):
    """在Tushare数据接口里，不管是旧版的一些接口还是Pro版的行情接口，都是以用户设定的end_date开始往前复权，跟所有行情软件或者财经网站上看到的前复权可能存在差异，
    因为所有这些都是以最近一个交易日开始往前复权的。比如今天是2018年10月26日，您想查2018年1月5日～2018年9月28日的前复权数据，Tushare是先查找9月28日的复权因子，
    从28日开始复权，而行情软件是从10月26日这天开始复权的。
    """
    def fetch_data():
        data = None
        try:
            time.sleep(0.002)
            #pro = get_pro()
            data = ts.pro_bar(
                api=pro,
                ts_code=str(name),
                asset='E',
                adj=adj,
                start_date=start,
                end_date=end,
                ma=ma,
                freq=freq,
                adjfactor=adjfactor,
                factors=['tor','vr']
            ).sort_index()
            print('fetch done: ' + str(name))
        except Exception as e:
            print(e)
            print('except when fetch data of ' + str(name))
            time.sleep(1)
            data = fetch_data()
        return data
    data = fetch_data()

    #data['date_stamp'] = data['trade_date'].apply(lambda x: cover_time(x))
    data['code'] = data['ts_code'].apply(lambda x: str(x)[0:6])
    data['fqtype'] = adj
    if type_ in ['json']:
        data_json = QA_util_to_json_from_pandas(data)
        return data_json
    elif type_ in ['pd', 'pandas']:
        data['date'] = pd.to_datetime(data['trade_date'], format='%Y%m%d')
        data = data.set_index('date', drop=False)
        data['date'] = data['date'].apply(lambda x: str(x)[0:10])
        return data


if __name__ == '__main__':          
    print(fetch_stock_list_tusharepro())
    print(fetch_stock_day_adj_fillna_from_tusharepro('2021-11-19'))
    print(fetch_get_stock_bar_hfq_one(name='600519.SH')) 
    print(fetch_get_stock_bar_hfq_one(name='830839.BJ'))
    print(fetch_get_stock_bar_hfq_one(name='430047.BJ'))
    pass





#coding=utf-8

from time import time

from rrshare.rqUtil.rqSingleton import Singleton_wraps

@Singleton_wraps
class rq_util_cache:
    '''简单的缓存系统
    把这个变量保存在内存里, 同时给它一个过期时间, 过期则失效.
    '''
    def __init__(self):
        '''初始化'''
        self.mem = {}
        self.time = {}

    def set(self, key, data, age=-1):
        '''保存键为key的值，存活时间为age秒'''
        self.mem[key] = data
        if age == -1:
            self.time[key] = -1
        else:
            self.time[key] = time() + age
        return True

    def get(self,key):
        '''获取键key对应的值'''
        if key in self.mem.keys():
            if self.time[key] == -1 or self.time[key] > time():
                return self.mem[key]
            else:
                self.delete(key)
                return None
        else:
            return None

    def delete(self,key):
        '''删除键为key的条目'''
        del self.mem[key]
        del self.time[key]
        return True

    def clear(self):
        '''清空所有缓存'''
        self.mem.clear()
        self.time.clear()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rrshare',
 'rrshare.RQSetting',
 'rrshare.rqFactor',
 'rrshare.rqFetch',
 'rrshare.rqSU',
 'rrshare.rqUpdate',
 'rrshare.rqUtil',
 'rrshare.rqWeb',
 'rrshare.sql']

package_data = \
{'': ['*'], 'rrshare': ['templates/*'], 'rrshare.rqWeb': ['templates/*']}

install_requires = \
['click>=7.1.2,<8.0.0',
 'easyquotation>=0.7.4,<0.8.0',
 'tushare>=1.2.62,<2.0.0',
 'zenlog>=1.1,<2.0']

entry_points = \
{'console_scripts': ['rrshare = rrshare:entry_point']}

setup_kwargs = {
    'name': 'rrshare',
    'version': '4.1.16',
    'description': 'stock data(from tusharepro) & analysis',
    'long_description': None,
    'author': 'rome',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from rrshare.rqUtil import rq_util_get_pre_trade_date

from rrshare.rqSU import (rq_save_swl_industry_list_stock_belong,  rq_save_swl_day_pg)
                

def record_swl_day(N=255):
    rq_save_swl_industry_list_stock_belong()
    rq_save_swl_day_pg(rq_util_get_pre_trade_date(n=N))

if __name__ == '__main__':
    #print(rq_util_get_pre_trade_date(n=255))
    record_swl_day(255)


#!/usr/bin/python
# conding:utf-8
# TODO
import time
import logging
from typing import List, Tuple, Union
import numpy as np
import pandas as pd
import pymysql
from sqlalchemy import create_engine
from rrshare.rqUtil.config_setting import setting
from rrshare.rqUtil.rqDate_trade import (rq_util_get_last_tradedate, rq_util_get_pre_trade_date)

logging.basicConfig(level=logging.DEBUG,format=' %(asctime)s-%(levelname)s-%(message)s ')

db_mysql ='rrshare'
user_mysql = 'root'
password_mysql = setting['MYSQL_PASSWORD']

logging.info(user_mysql)
logging.debug(password_mysql)


def conn_mysql(host="127.0.0.1", user=user_mysql,passwd=password_mysql,port=3306):
    return pymysql.connect(host=host, user=user,passwd=passwd,port=port,charset='utf8')
#logging.warning(conn_mysql())

def conn_mysqldb(db=db_mysql,host="127.0.0.1", user=user_mysql,passwd=password_mysql,port=3306):
    return pymysql.connect(host=host, user=user,passwd=passwd,database=db,port=port,charset='utf8')
#logging.info(conn_mysqldb('test_rrshare'))

def mysql_engine(db=db_mysql,host="127.0.0.1",user=user_mysql,passwd=password_mysql,port=3306):
    try:
        return create_engine(f'mysql+pymysql://{user}:{passwd}@{host}:{port}/{db}',encoding='utf8')
    except Exception as e:
        print(e)
#logging.warning(mysql_engine('test_rrshare'))


def mysql_conn(db=db_mysql):
    try:
        conn_sql = conn_mysql()
        cursor = conn_sql.cursor()
        #创建数据库，如果不存在
        sql = f"CREATE DATABASE IF NOT EXISTS {db}"
        #执行创建数据库
        cursor.execute(sql)
        logging.info(f'create mysql {db}')
    except AssertionError as error:
        print(error)
    else:
        try:
            conn = mysql_engine(db)
            logging.info(f'connect to mysql {db}')
            return conn
        except:
            logging.error(f'can not connect to mysql  {db}')
            

def mysql_create_table(sql, table_name, db=db_mysql):
    conn = conn_mysqldb(db) 
    try:
        cursor = conn.cursor()
        sql = sql.format(table_name)
        cursor.execute(sql)
        logging.info(f'create table {table_name} in datadase {db}')
    except Exception as e:
        print(e)


def write_to_mysql(df, db, table,if_exists='replace'):
    #与MySQL建立连接，
    conn = mysql_engine(db)
    #写入数据，table为表名，‘replace’表示如果同名表存在就替换掉
    try:
        pd.io.sql.to_sql(df,f"{table}", conn, if_exists=if_exists)
        print(f'写入数据库{db}表{table}, ok')
    except Exception as e:
        print(e)

def read_mysql_table(db,table,N):
    #与MySQL建立连接，
    conn = mysql_engine(db)
    #logging.info(conn)
    startTD = rq_util_get_pre_trade_date(rq_util_get_last_tradedate(),N)
    startTD = f'"{startTD}"'   ### very important !!!
    logging.debug(startTD)
    #get table from db's table
    try:
        #logging.info(f'try to read data from {db}.{table} ...... ')
        sql = f"SELECT * FROM {table} WHERE bob >= {startTD}"
        df = pd.read_sql(sql,conn)
        logging.info(f'read from database:{db} table:{table} N_tradedate \n {df.head()}')
        return df
    except Exception as e:
        print(e)

def read_mysql_sql(data: Union[str,list,tuple] = None, 
                  table: Union[str] = None, 
                  db: str = db_mysql
                  )  -> pd.DataFrame:
    try:
        conn = mysql_engine(db)
        sql = ''.join(['SELECT ', data,  ' FROM ', f'{table}'])
        logging.info(sql)
        df = pd.read_sql(sql, conn)
        logging.info(f'read sql from {db}/{table}\n {df}')
        return df
    except Exception as e:
        logging.error(e)
#read_mysql_sql('select name,close from realtime_price', 'realtime_price','test_rrshare' )


def read_mysql_select(secs, db, table,N):
    #与MySQL建立连接，
    conn = mysql_engine(db)
    startTD = rq_util_get_pre_trade_date(rq_util_get_last_tradedate(),N) 
    startTD = f'"{startTD}"'   ### very important !!!
    secs = ','.join(f"'{x}'" for x in secs) 
    logging.debug(secs) 
    try:
        print(f'try to read data from {db}.{table} ...... ')
        sql = f"SELECT * FROM {table} WHERE bob >= {startTD} AND symbol IN({secs})"
        df = pd.read_sql(sql,conn)
        logging.info(f'select date and code \n {df.tail()}')
        return df
    except Exception as e:
        print(e)


if __name__ == '__main__':
    """
    sql = '''CREATE TABLE stock_day(
        `trade_date` date NOT NULL,
        `code` varchar(45) NOT NULL,
        `open` decimal(20,2) DEFAULT NULL,
        `close` decimal(20,2) DEFAULT NULL,
        `high` decimal(20,2) DEFAULT NULL,
        `low` decimal(20,2) DEFAULT NULL,
        `volume` decimal(20) DEFAULT NULL,
        `amount` decimal(30,2) DEFAULT NULL,
        PRIMARY KEY (`trade_date`,`code`)
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;
        '''
    
    data = 'name , close'
    table = 'price' 
    sql = ''.join(['SELECT ', data,  ' FROM ', f'{table}'])
    logging.info(sql)
    """
    #mysql_conn('test_rrshare')
    #mysql_create_table(sql,'stock_day','test_rrshare')
    read_mysql_sql(data='name,pro_close,close,date,time',table='realtime_price',db='test_rrshare' )
    
    #logging.info(df)
    #write_to_mysql(df, 'test_rrshare','test')
    
    #read_mysql_table('gm','stock_day',3)
    #secs = ['SZSE.000001','SHSE.600519'] 
    #df = read_mysql_select(secs,'gm','stock_day',2)
    pass

    
